@@ -1,12 +1,12 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2007 Joe Wreschnig, Michael Urman, Iñigo Serna
 #           2009-2010 Steven Robertson
 #      2012,2013,2016 Nick Boultbee
 #           2009-2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk
 
@@ -47,7 +47,7 @@ class Preferences(qltk.UniqueWindow, EditDisplayPatternMixin):
     def __init__(self, browser):
         if self.is_not_unique():
             return
-        super(Preferences, self).__init__()
+        super().__init__()
         self.set_border_width(12)
         self.set_title(_("Album List Preferences"))
         self.set_default_size(420, 380)
