@@ -1,19 +1,27 @@
-# -*- coding: utf-8 -*-
 # Copyright 2011 Joe Wreschnig, Christoph Reiter
-#           2016 Nick Boultbee
+#      2013-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 import sys
 import bz2
 import itertools
+from functools import reduce
+from http.client import HTTPException
+from os.path import splitext
+from threading import Thread
+from typing import Dict, Collection, Callable, Iterable, Optional
+from urllib.request import urlopen
 
+import re
 from gi.repository import Gtk, GLib, Pango
+from senf import text2fsn
 
-from quodlibet.util.dprint import print_d
+from quodlibet.util.dprint import print_d, print_e
 
 import quodlibet
 from quodlibet import _
@@ -26,13 +34,11 @@ from quodlibet.formats.remote import RemoteFile
 from quodlibet.formats._audio import TAG_TO_SORT, MIGRATE, AudioFile
 from quodlibet.library import SongLibrary
 from quodlibet.query import Query
-from quodlibet.compat import reduce, urlopen
 from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.notif import Task
 from quodlibet.qltk import Icons, ErrorMessage, WarningMessage
-from quodlibet.util import copool, connect_destroy, sanitize_tags, \
-    connect_obj, escape
+from quodlibet.util import copool, connect_destroy, sanitize_tags, connect_obj
 from quodlibet.util.i18n import numeric_phrase
 from quodlibet.util.path import uri_is_valid
 from quodlibet.util.string import decode, encode
@@ -40,25 +46,24 @@ from quodlibet.util import print_w
 from quodlibet.qltk.views import AllTreeView
 from quodlibet.qltk.searchbar import SearchBarBox
 from quodlibet.qltk.completion import LibraryTagCompletion
-from quodlibet.qltk.x import MenuItem, Align, ScrolledWindow
-from quodlibet.qltk.x import SymbolicIconImage
-from quodlibet.qltk.menubutton import MenuButton
-from quodlibet.compat import text_type, iteritems, iterkeys
-
+from quodlibet.qltk.x import MenuItem, Align, ScrolledWindow, Button
 
 STATION_LIST_URL = \
-    "https://bitbucket.org/lazka/quodlibet/downloads/radiolist.bz2"
+    "https://quodlibet.github.io/radio/radiolist.bz2"
 STATIONS_FAV = os.path.join(quodlibet.get_user_dir(), "stations")
 STATIONS_ALL = os.path.join(quodlibet.get_user_dir(), "stations_all")
 
-# TODO: - Do the update in a thread
-#       - Ranking: reduce duplicate stations (max 3 URLs per station)
+# TODO: - Ranking: reduce duplicate stations (max 3 URLs per station)
 #                  prefer stations that match a genre?
 
 # Migration path for pickle
 sys.modules["browsers.iradio"] = sys.modules[__name__]
 
 
+class IRadioError(Exception):
+    pass
+
+
 class IRFile(RemoteFile):
     multisong = True
     can_add = False
@@ -89,11 +94,11 @@ class IRFile(RemoteFile):
         return base_call(key, *args, **kwargs)
 
     def __call__(self, key, *args, **kwargs):
-        base_call = super(IRFile, self).__call__
+        base_call = super().__call__
         return self.__get(base_call, key, *args, **kwargs)
 
     def get(self, key, *args, **kwargs):
-        base_call = super(IRFile, self).get
+        base_call = super().get
         return self.__get(base_call, key, *args, **kwargs)
 
     def write(self):
@@ -105,7 +110,7 @@ class IRFile(RemoteFile):
         if "title" in self:
             title = self["title"]
             del self["title"]
-        dump = super(IRFile, self).to_dump()
+        dump = super().to_dump()
         if title is not None:
             self["title"] = title
 
@@ -117,6 +122,10 @@ class IRFile(RemoteFile):
                 lines.append(encode(tag) + b"=" + encode(value))
         return b"\n".join(lines)
 
+    @property
+    def lyric_filename(self) -> Optional[str]:
+        return None
+
     def can_change(self, k=None):
         if self.streamsong:
             if k is None:
@@ -130,10 +139,11 @@ class IRFile(RemoteFile):
                 return k in self.__CAN_CHANGE
 
 
-def ParsePLS(file):
+def parse_pls(file) -> Collection[IRFile]:
     data = {}
 
-    lines = file.readlines()
+    lines = file.read().decode('utf-8', 'replace').splitlines()
+
     if not lines or "[playlist]" not in lines.pop(0):
         return []
 
@@ -147,15 +157,15 @@ def ParsePLS(file):
             if head.startswith("length") and val == "-1":
                 continue
             else:
-                data[head] = val.decode('utf-8', 'replace')
+                data[head] = val
 
     count = 1
     files = []
     warnings = []
     while True:
         if "file%d" % count in data:
-            filename = data["file%d" % count].encode('utf-8', 'replace')
-            if filename.lower()[-4:] in [".pls", ".m3u"]:
+            filename = text2fsn(data["file%d" % count])
+            if filename.lower()[-4:] in [".pls", ".m3u", "m3u8"]:
                 warnings.append(filename)
             else:
                 irf = IRFile(filename)
@@ -174,21 +184,19 @@ def ParsePLS(file):
         count += 1
 
     if warnings:
-        WarningMessage(
-            None, _("Unsupported file type"),
+        raise IRadioError(
             _("Station lists can only contain locations of stations, "
               "not other station lists or playlists. The following locations "
               "cannot be loaded:\n%s") %
-            "\n  ".join(map(util.escape, warnings))
-        ).run()
-
+            "\n  ".join(map(util.escape, warnings)))
     return files
 
 
-def ParseM3U(fileobj):
+def parse_m3u(fileobj) -> Collection[IRFile]:
     files = []
     pending_title = None
-    for line in fileobj:
+    lines = fileobj.read().decode('utf-8', 'replace').splitlines()
+    for line in lines:
         line = line.strip()
         if line.startswith("#EXTINF:"):
             try:
@@ -196,45 +204,59 @@ def ParseM3U(fileobj):
             except IndexError:
                 pending_title = None
         elif line.startswith("http"):
-            irf = IRFile(line)
+            irf = IRFile(text2fsn(line))
             if pending_title:
-                irf["title"] = pending_title.decode('utf-8', 'replace')
+                irf["title"] = pending_title
                 pending_title = None
             files.append(irf)
     return files
 
 
-def add_station(uri):
+def _get_stations_from(uri: str,
+                       on_done: Callable[[Iterable[IRFile], str], None])\
+        -> None:
     """Fetches the URI content and extracts IRFiles
-    Returns None in error, else a possibly filled list of stations"""
+       Called from thread - so no direct GTK+ interaction
+       :param uri: URI of station
+       :param on_done: a callback taking files when done (or none if errored)
+       """
 
-    irfs = []
+    with Task(_("Internet Radio"), _("Add stations")) as task:
+        irfs: Collection[IRFile] = []
+        GLib.idle_add(task.pulse)
+        if (uri.lower().endswith(".pls")
+                or uri.lower().endswith(".m3u")
+                or uri.lower().endswith(".m3u8")):
+            if not re.match('^([^/:]+)://', uri):
+                # Assume HTTP if no protocol given. See #2731
+                uri = 'http://' + uri
+                print_d("Assuming http: %s" % uri)
 
-    if uri.lower().endswith(".pls") or uri.lower().endswith(".m3u"):
-        try:
-            sock = urlopen(uri)
-        except EnvironmentError as err:
-            err = text_type(err)
-            print_d("Got %s from %s" % (uri, err))
-            ErrorMessage(None, _("Unable to add station"), escape(err)).run()
-            return None
-
-        if uri.lower().endswith(".pls"):
-            irfs = ParsePLS(sock)
-        elif uri.lower().endswith(".m3u"):
-            irfs = ParseM3U(sock)
-
-        sock.close()
-    else:
-        try:
-            irfs = [IRFile(uri)]
-        except ValueError as err:
-            ErrorMessage(None, _("Unable to add station"), err).run()
-
-    return irfs
+            # Error handling outside
+            sock = None
+            GLib.idle_add(task.pulse)
+            _fn, ext = splitext(uri.lower())
+            try:
+                sock = urlopen(uri, timeout=6)
+                if ext == ".pls":
+                    irfs = parse_pls(sock)
+                elif ext in (".m3u", ".m3u8"):
+                    irfs = parse_m3u(sock)
+                GLib.idle_add(task.pulse)
+            except IOError as e:
+                print_e(f"Couldn't download from {uri} ({e})")
+            finally:
+                if sock:
+                    sock.close()
+        else:
+            try:
+                irfs = [IRFile(uri)]
+            except ValueError as e:
+                print_e("Can't add URI %s" % uri, e)
+    on_done(irfs, uri)
 
 
-def download_taglist(callback, cofuncid, step=1024 * 10):
+def download_taglist(url, callback, cofuncid, step=1024 * 10):
     """Generator for loading the bz2 compressed tag list.
 
     Calls callback with the decompressed data or None in case of
@@ -245,11 +267,11 @@ def download_taglist(callback, cofuncid, step=1024 * 10):
             task.copool(cofuncid)
 
         try:
-            response = urlopen(STATION_LIST_URL)
-        except EnvironmentError:
+            response = urlopen(url)
+        except (EnvironmentError, HTTPException) as e:
+            print_e(f"Failed fetching from {url}", e)
             GLib.idle_add(callback, None)
             return
-
         try:
             size = int(response.info().get("content-length", 0))
         except ValueError:
@@ -257,8 +279,8 @@ def download_taglist(callback, cofuncid, step=1024 * 10):
 
         decomp = bz2.BZ2Decompressor()
 
-        data = ""
-        temp = ""
+        data = b""
+        temp = b""
         read = 0
         while temp or not data:
             read += len(temp)
@@ -282,7 +304,7 @@ def download_taglist(callback, cofuncid, step=1024 * 10):
         stations = None
         if data:
             stations = parse_taglist(data)
-
+        print_d(f"Got {len(stations or [])} station(s)")
         GLib.idle_add(callback, stations)
 
 
@@ -301,17 +323,20 @@ def parse_taglist(data):
     stations = []
     station = None
 
-    for l in data.split("\n"):
-        key = l.split("=")[0]
-        value = l.split("=", 1)[1]
+    for l in data.split(b"\n"):
+        if not l:
+            continue
+        key = l.split(b"=")[0]
+        value = l.split(b"=", 1)[1]
+        key = decode(key)
+        value = decode(value)
         if key == "uri":
             if station:
                 stations.append(station)
             station = IRFile(value)
             continue
 
-        value = decode(value)
-        san = sanitize_tags({key: value}, stream=True).items()
+        san = list(sanitize_tags({key: value}, stream=True).items())
         if not san:
             continue
 
@@ -320,8 +345,10 @@ def parse_taglist(data):
             key = "~#listenerpeak"
             value = int(value)
 
-        if isinstance(value, bytes):
-            value = value.decode("utf-8")
+        if not station:
+            continue
+
+        if isinstance(value, str):
             if value not in station.list(key):
                 station.add(key, value)
         else:
@@ -335,7 +362,7 @@ def parse_taglist(data):
 
 class AddNewStation(GetStringDialog):
     def __init__(self, parent):
-        super(AddNewStation, self).__init__(
+        super().__init__(
             parent, _("New Station"),
             _("Enter the location of an Internet radio station:"),
             button_label=_("_Add"), button_icon=Icons.LIST_ADD)
@@ -349,7 +376,7 @@ class AddNewStation(GetStringDialog):
                 return line
 
 
-class GenreFilter(object):
+class GenreFilter:
     STAR = ["genre", "organization"]
 
     # This probably needs improvements
@@ -359,8 +386,8 @@ class GenreFilter(object):
             "|(electr,house,techno,trance,/trip.?hop/,&(drum,n,bass),chill,"
             "dnb,minimal,/down(beat|tempo)/,&(dub,step))"),
         "rap": (_("Hip Hop / Rap"), "|(&(hip,hop),rap)"),
-        "oldies": (_("Oldies"), "|(/[2-9]0\S?s/,oldies)"),
-        "r&b": (_("R&B"), "/r(\&|n)b/"),
+        "oldies": (_("Oldies"), r"|(/[2-9]0\S?s/,oldies)"),
+        "r&b": (_("R&B"), r"/r(\&|n)b/"),
         "japanese": (_("Japanese"), "|(anime,jpop,japan,jrock)"),
         "indian": (_("Indian"), "|(bollywood,hindi,indian,bhangra)"),
         "religious": (
@@ -368,7 +395,7 @@ class GenreFilter(object):
             "|(religious,christian,bible,gospel,spiritual,islam)"),
         "charts": (_("Charts"), "|(charts,hits,top)"),
         "turkish": (_("Turkish"), "|(turkish,turkce)"),
-        "reggae": (_("Reggae / Dancehall"), "|(/reggae([^\w]|$)/,dancehall)"),
+        "reggae": (_("Reggae / Dancehall"), r"|(/reggae([^\w]|$)/,dancehall)"),
         "latin": (_("Latin"), "|(latin,salsa)"),
         "college": (_("College Radio"), "|(college,campus)"),
         "talk_news": (_("Talk / News"), "|(news,talk)"),
@@ -397,7 +424,7 @@ class GenreFilter(object):
     }
 
     # parsing all above takes 350ms on an atom, so only generate when needed
-    __CACHE = {}
+    __CACHE: Dict[str, Query] = {}
 
     def keys(self):
         return self.GENRES.keys()
@@ -421,7 +448,7 @@ class CloseButton(Gtk.Button):
         image = Gtk.Image(visible=True, can_focus=False,
                           icon_name="window-close-symbolic")
 
-        super(CloseButton, self).__init__(
+        super().__init__(
             visible=False, can_focus=True, image=image,
             relief=Gtk.ReliefStyle.NONE, valign=Gtk.Align.CENTER)
 
@@ -441,7 +468,7 @@ class QuestionBar(Gtk.InfoBar):
     RESPONSE_LOAD = 1
 
     def __init__(self):
-        super(QuestionBar, self).__init__()
+        super().__init__()
         self.connect("response", self.__response)
         self.set_message_type(Gtk.MessageType.QUESTION)
 
@@ -461,7 +488,6 @@ class QuestionBar(Gtk.InfoBar):
 
 
 class InternetRadio(Browser, util.InstanceTracker):
-
     __stations = None
     __fav_stations = None
     __librarian = None
@@ -508,6 +534,13 @@ class InternetRadio(Browser, util.InstanceTracker):
 
         klass.filters = None
 
+    def finalize(self, restored):
+        if not restored:
+            # Select "All Stations" by default
+            def sel_all(row):
+                return row[self.TYPE] == self.TYPE_ALL
+            self.view.select_by_func(sel_all, one=True)
+
     def __inhibit(self):
         self.view.get_selection().handler_block(self.__changed_sig)
 
@@ -518,8 +551,9 @@ class InternetRadio(Browser, util.InstanceTracker):
         if not self.instances():
             self._destroy()
 
-    def __init__(self, library):
-        super(InternetRadio, self).__init__(spacing=12)
+    def __init__(self, library, station_list_url: str = STATION_LIST_URL):
+        super().__init__(spacing=12)
+        self.station_list_url = station_list_url
         self.set_orientation(Gtk.Orientation.VERTICAL)
 
         if not self.instances():
@@ -534,20 +568,6 @@ class InternetRadio(Browser, util.InstanceTracker):
                                                  accel_group=self.accelerators)
         search.connect('query-changed', self.__filter_changed)
 
-        menu = Gtk.Menu()
-        new_item = MenuItem(_(u"_New Station…"), Icons.LIST_ADD)
-        new_item.connect('activate', self.__add)
-        menu.append(new_item)
-        update_item = MenuItem(_("_Update Stations"), Icons.VIEW_REFRESH)
-        update_item.connect('activate', self.__update)
-        menu.append(update_item)
-        menu.show_all()
-
-        button = MenuButton(
-            SymbolicIconImage(Icons.EMBLEM_SYSTEM, Gtk.IconSize.MENU),
-            arrow=True)
-        button.set_menu(menu)
-
         def focus(widget, *args):
             qltk.get_top_parent(widget).songlist.grab_focus()
         search.connect('focus-out', focus)
@@ -564,12 +584,10 @@ class InternetRadio(Browser, util.InstanceTracker):
         scrolled_window.add(view)
         model = Gtk.ListStore(int, str, str, str)
 
-        model.append(row=[self.TYPE_ALL, Icons.FOLDER, "__all",
-                          _("All Stations")])
+        model.append(row=[self.TYPE_ALL, Icons.FOLDER, "__all", _("All Stations")])
         model.append(row=[self.TYPE_SEP, Icons.FOLDER, "", ""])
-        #Translators: Favorite radio stations
-        model.append(row=[self.TYPE_FAV, Icons.FOLDER, "__fav",
-                          _("Favorites")])
+        # Translators: Favorite radio stations
+        model.append(row=[self.TYPE_FAV, Icons.FOLDER, "__fav", _("Favorites")])
         model.append(row=[self.TYPE_SEP, Icons.FOLDER, "", ""])
 
         filters = self.filters
@@ -612,8 +630,7 @@ class InternetRadio(Browser, util.InstanceTracker):
 
         box = Gtk.HBox(spacing=6)
         box.pack_start(search, True, True, 0)
-        box.pack_start(button, False, True, 0)
-        self._searchbox = Align(box, left=0, right=6, top=6)
+        self._searchbox = Align(box, left=0, right=6, top=0)
         self._searchbox.show_all()
 
         def qbar_response(infobar, response_id):
@@ -622,25 +639,38 @@ class InternetRadio(Browser, util.InstanceTracker):
 
         self.qbar = QuestionBar()
         self.qbar.connect("response", qbar_response)
-        if self._is_library_empty():
+        if not self.has_stations:
             self.qbar.show()
 
         pane = qltk.ConfigRHPaned("browsers", "internetradio_pos", 0.4)
-        pane.show()
-        pane.pack1(scrolled_window, resize=False, shrink=False)
+        vb = Gtk.VBox(spacing=0)
+        vb.pack_start(scrolled_window, True, True, 0)
+        fb = Gtk.FlowBox()
+        fb.set_column_spacing(3)
+        fb.set_homogeneous(True)
+        new_station = Button(_(u"_Add Station…"), Icons.LIST_ADD)
+        new_station.connect('clicked', self.__add)
+        self._update_button = Button(_("_Update Stations"), Icons.VIEW_REFRESH)
+        self._update_button.connect('clicked', self.__update)
+        fb.insert(new_station, 1)
+        fb.insert(self._update_button, 2)
+        vb.pack_end(Align(fb, left=3), False, False, 3)
+        pane.pack1(vb, resize=False, shrink=False)
+        pane.show_all()
+
         songbox = Gtk.VBox(spacing=6)
         songbox.pack_start(self._searchbox, False, True, 0)
         self._songpane_container = Gtk.VBox()
-        self._songpane_container.show()
         songbox.pack_start(self._songpane_container, True, True, 0)
         songbox.pack_start(self.qbar, False, True, 0)
-        songbox.show()
+        songbox.show_all()
         pane.pack2(songbox, resize=True, shrink=False)
         self.pack_start(pane, True, True, 0)
         self.show()
 
-    def _is_library_empty(self):
-        return not len(self.__stations) and not len(self.__fav_stations)
+    @property
+    def has_stations(self) -> bool:
+        return bool(len(self.__stations or []) + len(self.__fav_stations or []))
 
     def pack(self, songpane):
         container = Gtk.VBox()
@@ -654,7 +684,7 @@ class InternetRadio(Browser, util.InstanceTracker):
 
     def __update(self, *args):
         self.qbar.hide()
-        copool.add(download_taglist, self.__update_done,
+        copool.add(download_taglist, self.station_list_url, self.__update_done,
                    cofuncid="radio-load", funcid="radio-load")
 
     def __update_done(self, stations):
@@ -682,7 +712,7 @@ class InternetRadio(Browser, util.InstanceTracker):
 
         # keep at most 2 URLs for each group
         stations = []
-        for key, sub in iteritems(groups):
+        for key, sub in groups.items():
             sub.sort(key=lambda s: s.get("~#listenerpeak", 0), reverse=True)
             stations.extend(sub[:2])
 
@@ -690,7 +720,7 @@ class InternetRadio(Browser, util.InstanceTracker):
         all_ = [self.filters.query(k) for k in self.filters.keys()]
         assert all_
         anycat_filter = reduce(lambda x, y: x | y, all_)
-        stations = filter(anycat_filter.search, stations)
+        stations = list(filter(anycat_filter.search, stations))
 
         # remove listenerpeak
         for s in stations:
@@ -699,11 +729,11 @@ class InternetRadio(Browser, util.InstanceTracker):
         # update the libraries
         stations = dict(((s.key, s) for s in stations))
         # don't add ones that are in the fav list
-        for fav in iterkeys(self.__fav_stations):
+        for fav in self.__fav_stations.keys():
             stations.pop(fav, None)
 
         # separate
-        o, n = set(iterkeys(self.__stations)), set(stations)
+        o, n = set(self.__stations.keys()), set(stations)
         to_add, to_change, to_remove = n - o, o & n, o - n
         del o, n
 
@@ -714,7 +744,7 @@ class InternetRadio(Browser, util.InstanceTracker):
             # clear everything except stats
             AudioFile.reload(old)
             # add new metadata except stats
-            for k in (x for x in iterkeys(new) if x not in MIGRATE):
+            for k in (x for x in new.keys() if x not in MIGRATE):
                 old[k] = new[k]
 
         to_add = [stations.pop(k) for k in to_add]
@@ -744,7 +774,7 @@ class InternetRadio(Browser, util.InstanceTracker):
         return libs
 
     def __get_selection_filter(self):
-        """Retuns a filter object for the current selection or None
+        """Returns a filter object for the current selection or None
         if nothing should be filtered"""
 
         selection = self.view.get_selection()
@@ -776,6 +806,9 @@ class InternetRadio(Browser, util.InstanceTracker):
 
         return filter_
 
+    def unfilter(self):
+        self.filter_text("")
+
     def __add_fav(self, songs):
         songs = [s for s in songs if s in self.__stations]
         type(self).__librarian.move(
@@ -790,28 +823,36 @@ class InternetRadio(Browser, util.InstanceTracker):
         parent = qltk.get_top_parent(self)
         uri = (AddNewStation(parent).run(clipboard=True) or "").strip()
         if uri != "":
-            self.__add_station(uri)
+            self.__add_stations_from(uri)
 
-    def __add_station(self, uri):
-        irfs = add_station(uri)
-        if irfs is None:
-            # Error, rather than empty list
-            return
+    def __add_stations(self, irfs: Collection[IRFile], uri: str) -> None:
+        print_d(f"Got {len(irfs)} station(s) from {uri}")
+        assert self.__fav_stations is not None
         if not irfs:
-            ErrorMessage(
-                None, _("No stations found"),
+            msg = ErrorMessage(
+                self, _("No stations found"),
                 _("No Internet radio stations were found at %s.") %
-                util.escape(uri)).run()
+                util.escape(uri))
+            msg.run()
             return
 
-        irfs = set(irfs) - set(self.__fav_stations)
-        if not irfs:
-            WarningMessage(
-                None, _("Unable to add station"),
-                _("All stations listed are already in your library.")).run()
-
+        fav_uris = {af("~uri") for af in self.__fav_stations}
+        irfs = {af for af in irfs if af("~uri") not in fav_uris}
         if irfs:
+            print_d(f"Adding {irfs} to favourites")
             self.__fav_stations.add(irfs)
+        else:
+            message = WarningMessage(
+                self,
+                _("Nothing to add"),
+                _("All stations listed are already in your library."))
+            message.run()
+
+    def __add_stations_from(self, uri: str) -> None:
+        def on_done(irfs: Iterable[IRFile], uri: str):
+            GLib.idle_add(self.__add_stations, irfs, uri)
+            print_d("Quitting thread")
+        Thread(target=_get_stations_from, args=(uri, on_done)).start()
 
     def Menu(self, songs, library, items):
         in_fav = False
@@ -836,13 +877,13 @@ class InternetRadio(Browser, util.InstanceTracker):
 
         items.append(iradio_items)
         menu = SongsMenu(self.__librarian, songs, playlists=False, remove=True,
-                         queue=False, devices=False, items=items)
+                         queue=False, items=items)
         return menu
 
     def restore(self):
         text = config.gettext("browsers", "query_text")
         self.__searchbar.set_text(text)
-        if Query.is_parsable(text):
+        if Query(text).is_parsable:
             self.__filter_changed(self.__searchbar, text, restore=True)
 
         keys = config.get("browsers", "radio").splitlines()
@@ -875,7 +916,7 @@ class InternetRadio(Browser, util.InstanceTracker):
 
     def filter_text(self, text):
         self.__searchbar.set_text(text)
-        if Query.is_parsable(text):
+        if Query(text).is_parsable:
             self.__filter_changed(self.__searchbar, text)
             self.activate()
 
@@ -928,12 +969,12 @@ class InternetRadio(Browser, util.InstanceTracker):
         self.view.set_cursor(path)
         self.view.scroll_to_cell(path, use_align=True, row_align=0.5)
 
-    def status_text(self, count, time=None):
-        return numeric_phrase("%(count)d station", "%(count)d stations",
-                              count, 'count')
+    def status_text(self, count: int, time: Optional[str] = None) -> str:
+        return numeric_phrase("%(count)d station", "%(count)d stations", count, 'count')
 
 
 from quodlibet import app
+
 if not app.player or app.player.can_play_uri("http://"):
     browsers = [InternetRadio]
 else:
