@@ -1,5 +1,6 @@
 # Copyright 2005 Joe Wreschnig, Michael Urman
 #           2012, 2013 Christoph Reiter
+#                 2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -7,12 +8,13 @@
 # (at your option) any later version.
 
 import contextlib
+import sys
+import os
 
 from gi.repository import Gtk, Gdk, GObject, Pango, GLib
 import cairo
-from senf import environ
 
-from quodlibet import _, print_e
+from quodlibet import _, print_e, util
 from quodlibet import config
 from quodlibet.qltk import get_top_parent, is_accel, is_wayland, gtk_version, \
     menu_popup, get_primary_accel_mod
@@ -39,6 +41,8 @@ class TreeViewHints(Gtk.Window):
          'enter-notify-event', 'leave-notify-event'],
         'override')
 
+    __empty_region = cairo.Region(cairo.RectangleInt())
+
     def __init__(self):
         try:
             # gtk+ 3.20
@@ -205,7 +209,7 @@ class TreeViewHints(Gtk.Window):
         # get the renderer at the mouse position and get the xpos/width
         renderers = col.get_cells()
         pos = sorted(zip(map(col.cell_get_position, renderers), renderers))
-        pos = list(filter(lambda p: p[0][0] < cellx, pos))
+        pos = [p for p in pos if p[0][0] < cellx]
         if not pos:
             self.__undisplay()
             return False
@@ -351,6 +355,13 @@ class TreeViewHints(Gtk.Window):
         set_text(label)
         self.set_size_request(w, h)
 
+        # Set region on this window for which to receive mouse events to the
+        # empty region. Mouse events will be passed to the window below the
+        # tooltip. The Gdk implementation for win32 does not support this, which
+        # leads to events not being received in either window.
+        if sys.platform != "win32":
+            self.input_shape_combine_region(self.__empty_region)
+
         window = self.get_window()
         if self.get_visible() and window:
             window.move_resize(x, y, w, h)
@@ -618,8 +629,7 @@ class BaseView(Gtk.TreeView):
     __gsignals__: GSignals = {
         # like the tree selection changed signal but doesn't emit twice in case
         # a row is activated
-        'selection-changed': (
-            GObject.SignalFlags.RUN_LAST, None, (object, )),
+        'selection-changed': (GObject.SignalFlags.RUN_LAST, None, (object, )),
     }
 
     def __init__(self, *args, **kwargs):
@@ -855,7 +865,7 @@ class BaseView(Gtk.TreeView):
 
     @contextlib.contextmanager
     def without_model(self):
-        """Conext manager which removes the model from the view
+        """Context manager which removes the model from the view
         and adds it back afterwards.
 
         Tries to preserve all state that gets reset on a model change.
@@ -923,7 +933,7 @@ class DragIconTreeView(BaseView):
         layout = None
         if len(paths) > max_rows:
             more = _(u"and %d more…") % (len(paths) - max_rows)
-            more = "<i>%s</i>" % more
+            more = util.italic(more)
             layout = self.create_pango_layout("")
             layout.set_markup(more)
             layout.set_alignment(Pango.Alignment.CENTER)
@@ -1144,7 +1154,7 @@ class HintedTreeView(BaseView):
         display scroll bars instead for example.
         """
 
-        if "QUODLIBET_NO_HINTS" in environ:
+        if "QUODLIBET_NO_HINTS" in os.environ:
             return False
 
         return not config.state('disable_hints')
@@ -1154,7 +1164,7 @@ class _TreeViewColumnLabel(Gtk.Label):
     """A label which fades  into the background at the end; for use
     only in TreeViewColumns.
 
-    The hackery with using the parents allocation is needed because
+    The hackery with using the parent's allocation is needed because
     the label always gets the allocation it has requested, ignoring
     the actual width of the column header.
     """
