@@ -1,18 +1,22 @@
-# -*- coding: utf-8 -*-
 # Copyright 2014 Christoph Reiter <reiter.christoph@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of version 2 of the GNU General Public License as
-# published by the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import re
 import shlex
+from typing import Dict, Tuple, Callable
+
+from senf import bytes2fsn, fsn2bytes
 
 from quodlibet import const
+from quodlibet.util import print_d, print_w
 from .tcpserver import BaseTCPServer, BaseTCPConnection
 
 
-class AckError(object):
+class AckError:
     NOT_LIST = 1
     ARG = 2
     PASSWORD = 3
@@ -27,7 +31,7 @@ class AckError(object):
     EXIST = 56
 
 
-class Permissions(object):
+class Permissions:
     PERMISSION_NONE = 0
     PERMISSION_READ = 1
     PERMISSION_ADD = 2
@@ -48,14 +52,12 @@ TAG_MAPPING = [
     (u"AlbumArtistSort", "albumartistsort"),
     (u"Title", "title"),
     (u"Track", "tracknumber"),
-    (u"Name", ""),
     (u"Genre", "genre"),
     (u"Date", "~year"),
     (u"Composer", "composer"),
     (u"Performer", "performer"),
     (u"Comment", "commend"),
     (u"Disc", "discnumber"),
-    (u"Time", "~#length"),
     (u"Name", "~basename"),
     (u"MUSICBRAINZ_ARTISTID", "musicbrainz_artistid"),
     (u"MUSICBRAINZ_ALBUMID", "musicbrainz_albumid"),
@@ -69,15 +71,7 @@ def format_tags(song):
 
     lines = []
     for mpd_key, ql_key in TAG_MAPPING:
-        if not ql_key:
-            continue
-
-        if ql_key.startswith("~#"):
-            value = song(ql_key, None)
-            if value is not None:
-                value = str(value)
-        else:
-            value = song.comma(ql_key) or None
+        value = song.comma(ql_key) or None
 
         if value is not None:
             lines.append(u"%s: %s" % (mpd_key, value))
@@ -97,18 +91,18 @@ def parse_command(line):
 
     assert isinstance(line, bytes)
 
-    parts = re.split("[ \\t]+", line, maxsplit=1)
+    parts = re.split(b"[ \\t]+", line, maxsplit=1)
     if not parts:
         raise ParseError("empty command")
     command = parts[0]
 
     if len(parts) > 1:
-        lex = shlex.shlex(parts[1], posix=True)
+        lex = shlex.shlex(bytes2fsn(parts[1], "utf-8"), posix=True)
         lex.whitespace_split = True
         lex.commenters = ""
         lex.quotes = "\""
         lex.whitespace = " \t"
-        args = list(lex)
+        args = [fsn2bytes(a, "utf-8") for a in lex]
     else:
         args = []
 
@@ -128,7 +122,7 @@ def parse_command(line):
     return command, dec_args
 
 
-class MPDService(object):
+class MPDService:
     """This is the actual shared MPD service which the clients talk to"""
 
     version = (0, 17, 0)
@@ -151,7 +145,7 @@ class MPDService(object):
         def options_changed(*args):
             self.emit_changed("options")
 
-        self._options.connect("notify::random", options_changed)
+        self._options.connect("notify::shuffle", options_changed)
         self._options.connect("notify::repeat", options_changed)
         self._options.connect("notify::single", options_changed)
 
@@ -207,7 +201,7 @@ class MPDService(object):
 
     def flush_idle(self):
         flushed = []
-        for conn, subs in self._idle_subscriptions.iteritems():
+        for conn, subs in self._idle_subscriptions.items():
             # figure out which subsystems to report for each connection
             queued = self._idle_queue[conn]
             if subs:
@@ -232,15 +226,12 @@ class MPDService(object):
         self._idle_subscriptions.pop(connection, None)
 
     def emit_changed(self, subsystem):
-        for conn, subs in self._idle_queue.iteritems():
+        for conn, subs in self._idle_queue.items():
             subs.add(subsystem)
         self.flush_idle()
 
     def play(self):
-        if not self._app.player.song:
-            self._app.player.reset()
-        else:
-            self._app.player.paused = False
+        self._app.player.playpause()
 
     def playid(self, songid):
         self.play()
@@ -280,13 +271,13 @@ class MPDService(object):
     def setvol(self, value):
         """value: 0..100"""
 
-        self._app.player.volume = (value / 100.0) ** 3.0
+        self._app.player.volume = value / 100.0
 
     def repeat(self, value):
         self._options.repeat = value
 
     def random(self, value):
-        self._options.random = value
+        self._options.shuffle = value
 
     def single(self, value):
         self._options.single = value
@@ -318,9 +309,9 @@ class MPDService(object):
             state = "stop"
 
         status = [
-            ("volume", int((app.player.volume ** (1.0 / 3.0)) * 100)),
+            ("volume", int(app.player.volume * 100)),
             ("repeat", int(self._options.repeat)),
-            ("random", int(self._options.random)),
+            ("random", int(self._options.shuffle)),
             ("single", int(self._options.single)),
             ("consume", 0),
             ("playlist", self._pl_ver),
@@ -330,6 +321,10 @@ class MPDService(object):
         ]
 
         if info:
+            status.append(("audio", "%d:%d:%d" % (
+                info("~#samplerate") or 0,
+                info("~#bitdepth") or 0,
+                info("~#channels") or 0)))
             total_time = int(info("~#length"))
             elapsed_time = int(app.player.get_position() / 1000)
             elapsed_exact = "%1.3f" % (app.player.get_position() / 1000.0)
@@ -355,6 +350,7 @@ class MPDService(object):
         parts = []
         parts.append(u"file: %s" % info("~filename"))
         parts.append(format_tags(info))
+        parts.append(u"Time: %d" % int(info("~#length")))
         parts.append(u"Pos: %d" % 0)
         parts.append(u"Id: %d" % self._get_id(info))
 
@@ -388,7 +384,7 @@ class MPDServer(BaseTCPServer):
     def __init__(self, app, config, port):
         self._app = app
         self._config = config
-        super(MPDServer, self).__init__(port, MPDConnection, const.DEBUG)
+        super().__init__(port, MPDConnection, const.DEBUG)
 
     def handle_init(self):
         print_d("Creating the MPD service")
@@ -420,8 +416,8 @@ class MPDConnection(BaseTCPConnection):
         self.service = service
         service.add_connection(self)
 
-        str_version = ".".join(map(str, service.version))
-        self._buf = bytearray("OK MPD %s\n" % str_version)
+        str_version = u".".join(map(str, service.version))
+        self._buf = bytearray((u"OK MPD %s\n" % str_version).encode("utf-8"))
         self._read_buf = bytearray()
 
         # begin - command processing state
@@ -495,7 +491,7 @@ class MPDConnection(BaseTCPConnection):
         """Returns the next line from the read buffer or None"""
 
         try:
-            index = self._read_buf.index("\n")
+            index = self._read_buf.index(b"\n")
         except ValueError:
             return None
 
@@ -506,10 +502,10 @@ class MPDConnection(BaseTCPConnection):
     def write_line(self, line):
         """Writes a line to the client"""
 
-        assert isinstance(line, unicode)
+        assert isinstance(line, str)
         self.log(u"<- " + repr(line))
 
-        self._buf.extend(line.encode("utf-8", errors="replace") + "\n")
+        self._buf.extend(line.encode("utf-8", errors="replace") + b"\n")
 
     def ok(self):
         self.write_line(u"OK")
@@ -586,7 +582,7 @@ class MPDConnection(BaseTCPConnection):
         elif do_ack:
             self.ok()
 
-    _commands = {}
+    _commands: Dict[str, Tuple[Callable, bool, int]] = {}
 
     @classmethod
     def Command(cls, name, ack=True, permission=Permissions.PERMISSION_ADMIN):
@@ -836,14 +832,13 @@ def _cmd_outputs(conn, service, args):
 @MPDConnection.Command("commands", permission=Permissions.PERMISSION_NONE)
 def _cmd_commands(conn, service, args):
     for name in conn.list_commands():
-        conn.write_line(u"command: " + unicode(name))
+        conn.write_line(u"command: " + str(name))
 
 
 @MPDConnection.Command("tagtypes")
 def _cmd_tagtypes(conn, service, args):
     for mpd_key, ql_key in TAG_MAPPING:
-        if ql_key:
-            conn.write_line(mpd_key)
+        conn.write_line(mpd_key)
 
 
 @MPDConnection.Command("lsinfo")
