@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig
-#     2012, 2016 Nick Boultbee
+#      2012-2022 Nick Boultbee
 #           2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -13,11 +12,14 @@ Librarians for libraries.
 """
 
 import itertools
+from typing import Dict, Iterable, Iterator, Generator
 
 from gi.repository import GObject
 
-from quodlibet.util.dprint import print_d
-from quodlibet.compat import itervalues
+from quodlibet.library.base import Library
+from quodlibet.library.playlist import PlaylistLibrary
+from quodlibet.util.dprint import print_d, print_w
+from senf import fsnative
 
 
 class Librarian(GObject.GObject):
@@ -40,14 +42,14 @@ class Librarian(GObject.GObject):
     }
 
     def __init__(self):
-        super(Librarian, self).__init__()
-        self.libraries = {}
+        super().__init__()
+        self.libraries: Dict[str, Library] = {}
         self.__signals = {}
 
-    def destroy(self):
+    def destroy(self) -> None:
         pass
 
-    def register(self, library, name):
+    def register(self, library: Library, name: str) -> None:
         """Register a library with this librarian."""
         if name in self.libraries or name in self.__signals:
             raise ValueError("library %r is already active" % name)
@@ -58,7 +60,7 @@ class Librarian(GObject.GObject):
         self.libraries[name] = library
         self.__signals[library] = [added_sig, removed_sig, changed_sig]
 
-    def _unregister(self, library, name):
+    def _unregister(self, library: Library, name: str) -> None:
         # This function, unlike register, should be private.
         # Libraries get unregistered at the discretion of the
         # librarian, not the libraries.
@@ -72,26 +74,26 @@ class Librarian(GObject.GObject):
     # a case where many libraries fire a signal at the same time (or
     # one fires a signal often).
 
-    def __changed(self, library, items):
+    def __changed(self, _library: Library, items: Iterable) -> None:
         self.emit('changed', items)
 
-    def __added(self, library, items):
+    def __added(self, _library: Library, items: Iterable) -> None:
         self.emit('added', items)
 
-    def __removed(self, library, items):
+    def __removed(self, _library: Library, items: Iterable) -> None:
         self.emit('removed', items)
 
-    def changed(self, items):
+    def changed(self, items: Iterable) -> None:
         """Triage the items and inform their real libraries."""
 
-        for library in itervalues(self.libraries):
+        for library in self.libraries.values():
             in_library = set(item for item in items if item in library)
             if in_library:
                 library._changed(in_library)
 
     def __getitem__(self, key):
         """Find a item given its key."""
-        for library in itervalues(self.libraries):
+        for library in self.libraries.values():
             try:
                 return library[key]
             except KeyError:
@@ -105,24 +107,24 @@ class Librarian(GObject.GObject):
         except KeyError:
             return default
 
-    def remove(self, items):
+    def remove(self, items: Iterable):
         """Remove items from all libraries."""
-        for library in itervalues(self.libraries):
+        for library in self.libraries.values():
             library.remove(items)
 
-    def __contains__(self, item):
+    def __contains__(self, item) -> bool:
         """Check if a key or item is in the library."""
-        for library in itervalues(self.libraries):
+        for library in self.libraries.values():
             if item in library:
                 return True
         else:
             return False
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator:
         """Iterate over all items in all libraries."""
-        return itertools.chain(*itervalues(self.libraries))
+        return itertools.chain(*self.libraries.values())
 
-    def move(self, items, from_, to):
+    def move(self, items: Iterable, from_: Library, to: Library) -> None:
         """Move items from one library to another.
 
         This causes 'removed' signals on the from library, and 'added'
@@ -138,13 +140,28 @@ class Librarian(GObject.GObject):
             from_.handler_unblock(self.__signals[from_][1])
             to.handler_unblock(self.__signals[to][0])
 
+    def move_root(self, old_root: fsnative, new_root: fsnative, **kwargs) -> Generator:
+        if old_root == new_root:
+            print_d("Not moving to same root")
+            return
+        for library in self.libraries.values():
+            if hasattr(library, "move_root"):
+                yield from library.move_root(old_root, new_root, **kwargs)
+
+    def remove_roots(self, old_roots: Iterable[fsnative]) -> Generator:
+        if not old_roots:
+            return
+        for library in self.libraries.values():
+            if hasattr(library, "remove_roots"):
+                yield from library.remove_roots(old_roots)
+
 
 class SongLibrarian(Librarian):
     """A librarian for SongLibraries."""
 
     def tag_values(self, tag):
         """Return a set of all values for the given tag."""
-        return {value for lib in itervalues(self.libraries)
+        return {value for lib in self.libraries.values()
                 for value in lib.tag_values(tag)}
 
     def rename(self, song, newname, changed=None):
@@ -159,8 +176,8 @@ class SongLibrarian(Librarian):
         # the call for future libraries because the item's key has
         # changed. So, it needs to reimplement the method.
         re_add = []
-        print_d("Renaming %r to %r" % (song.key, newname), self)
-        for library in itervalues(self.libraries):
+        print_d(f"Renaming {song.key!r} to {newname!r}")
+        for library in self.libraries.values():
             try:
                 del library._contents[song.key]
             except KeyError:
@@ -173,7 +190,6 @@ class SongLibrarian(Librarian):
             if changed is None:
                 library._changed({song})
             else:
-                print_d("Delaying changed signal for %r." % library, self)
                 changed.add(song)
 
     def reload(self, item, changed=None, removed=None):
@@ -186,8 +202,8 @@ class SongLibrarian(Librarian):
         """
 
         had_item = []
-        print_d("Reloading %r" % item.key, self)
-        for library in itervalues(self.libraries):
+        print_d(f"Reloading {item.key!r}")
+        for library in self.libraries.values():
             try:
                 del library._contents[item.key]
             except KeyError:
@@ -219,3 +235,15 @@ class SongLibrarian(Librarian):
                     library.emit('changed', {item})
             else:
                 changed.add(item)
+
+    @property
+    def playlists(self):
+        for lib in self.libraries.values():
+            if isinstance(lib, PlaylistLibrary):
+                return lib
+            try:
+                return lib.playlists
+            except AttributeError:
+                pass
+        print_w(f"No playlist library found: {self.libraries}")
+        raise ValueError("No playlists library found")
