@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
 # Copyright 2014 Christoph Reiter <reiter.christoph@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of version 2 of the GNU General Public License as
-# published by the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 
@@ -13,10 +13,10 @@ if os.name == "nt":
     raise PluginNotSupportedError
 
 import socket
-import threading
 
-from gi.repository import Gtk, GLib
+from gi.repository import Gtk
 
+from quodlibet import _
 from quodlibet.plugins import PluginConfigMixin
 from quodlibet.plugins.events import EventPlugin
 from quodlibet import app
@@ -24,14 +24,16 @@ from quodlibet import qltk
 from quodlibet import config
 from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk import Icons
+from quodlibet.util import print_w, print_d
+from quodlibet.util.thread import call_async, Cancellable
 
 from .main import MPDServer
 from .tcpserver import ServerError
 from .avahi import AvahiService, AvahiError
 
 
-def fill_ip(entry):
-    """Fill GtkEntry with the local IP. Can be called from a thread."""
+def fetch_local_ip():
+    """Returns a guess for the local IP"""
 
     try:
         s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
@@ -40,12 +42,7 @@ def fill_ip(entry):
         s.close()
     except EnvironmentError:
         addr = "?.?.?.?"
-
-    def idle_fill():
-        if entry.get_realized():
-            entry.set_text(addr)
-
-    GLib.idle_add(idle_fill)
+    return addr
 
 
 DEFAULT_PORT = 6600
@@ -78,7 +75,7 @@ class MPDServerPlugin(EventPlugin, PluginConfigMixin):
 
         label = Gtk.Label(label=_("_Port:"), use_underline=True)
         label.set_alignment(0.0, 0.5)
-        table.attach(label, 0, 1, 0, 1,
+        table.attach(label, 0, 1, 1, 2,
                      xoptions=Gtk.AttachOptions.FILL |
                      Gtk.AttachOptions.SHRINK)
 
@@ -96,7 +93,7 @@ class MPDServerPlugin(EventPlugin, PluginConfigMixin):
             try:
                 port_num = int(entry.get_text())
             except ValueError as e:
-                print_w(str(e))
+                print_w(e)
             else:
                 if get_port_num() != port_num:
                     set_port_num(port_num)
@@ -105,7 +102,7 @@ class MPDServerPlugin(EventPlugin, PluginConfigMixin):
         entry.connect_after("activate", port_activate)
         entry.connect_after("focus-out-event", port_activate)
 
-        table.attach(entry, 1, 2, 0, 1)
+        table.attach(entry, 1, 2, 1, 2)
 
         port_revert = Gtk.Button()
         port_revert.add(Gtk.Image.new_from_icon_name(
@@ -117,11 +114,11 @@ class MPDServerPlugin(EventPlugin, PluginConfigMixin):
 
         port_revert.connect("clicked", port_revert_cb, entry)
         table.attach(
-            port_revert, 2, 3, 0, 1, xoptions=Gtk.AttachOptions.SHRINK)
+            port_revert, 2, 3, 1, 2, xoptions=Gtk.AttachOptions.SHRINK)
 
         label = Gtk.Label(label=_("Local _IP:"), use_underline=True)
         label.set_alignment(0.0, 0.5)
-        table.attach(label, 0, 1, 1, 2,
+        table.attach(label, 0, 1, 0, 1,
                      xoptions=Gtk.AttachOptions.FILL |
                      Gtk.AttachOptions.SHRINK)
 
@@ -137,12 +134,16 @@ class MPDServerPlugin(EventPlugin, PluginConfigMixin):
 
         table.attach(entry, 1, 3, 2, 3)
 
-        entry = UndoEntry()
-        entry.set_text("...")
-        entry.set_editable(False)
-        table.attach(entry, 1, 3, 1, 2)
+        label = Gtk.Label()
+        label.set_padding(6, 6)
+        label.set_alignment(0.0, 0.5)
+        label.set_selectable(True)
+        label.set_label("...")
+        table.attach(label, 1, 3, 0, 1)
 
-        threading.Thread(target=fill_ip, args=(entry,)).start()
+        cancel = Cancellable()
+        label.connect("destroy", lambda *x: cancel.cancel())
+        call_async(fetch_local_ip, cancel, label.set_label)
 
         box = Gtk.VBox(spacing=12)
 
@@ -150,9 +151,9 @@ class MPDServerPlugin(EventPlugin, PluginConfigMixin):
         clients.set_padding(6, 6)
         clients.set_markup(u"""\
 \u2022 <a href="https://play.google.com/store/apps/details?id=com.\
-namelessdev.mpdroid">MPDroid 1.06</a> (Android)<small>
-
-</small>\u2022 <a href="http://sonata.berlios.de/">Sonata 1.6</a> (Linux)\
+namelessdev.mpdroid">MPDroid</a> (Android)
+\u2022 <a href="https://play.google.com/store/apps/details?id=org.\
+gateshipone.malp">M.A.L.P.</a> (Android)
 """)
         clients.set_alignment(0, 0)
 
@@ -176,7 +177,7 @@ namelessdev.mpdroid">MPDroid 1.06</a> (Android)<small>
         try:
             self._server.start()
         except ServerError as e:
-            print_w(str(e))
+            print_w(e)
 
     def _disable_server(self):
         print_d("Stopping MPD server")
@@ -190,7 +191,7 @@ namelessdev.mpdroid">MPDroid 1.06</a> (Android)<small>
         try:
             self._avahi.register(app.name, port_num, "_mpd._tcp")
         except AvahiError as e:
-            print_w(str(e))
+            print_w(e)
 
     def enabled(self):
         self._enable_server()
