@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # QLScrobbler: an Audioscrobbler client plugin for Quod Libet.
 # version 0.11
 # (C) 2005-2016 by Joshua Kwan <joshk@triplehelix.org>,
@@ -6,7 +5,7 @@
 #                  Franz Pletyz <fpletz@franz-pletz.org>,
 #                  Nicholas J. Michalek <djphazer@gmail.com>,
 #                  Steven Robertson <steven@strobe.cc>
-#                  Nick Boultbee <nick.boultbee@gmail.com>
+#     2012-2022    Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +15,9 @@
 import os
 import threading
 import time
+from typing import Dict, List
 from hashlib import md5
+from urllib.parse import urlencode
 
 from gi.repository import Gtk, GLib
 
@@ -30,13 +31,11 @@ from quodlibet.plugins import PluginConfig
 from quodlibet.qltk.entry import ValidatingEntry, UndoEntry
 from quodlibet.qltk.msg import Message
 from quodlibet.qltk import Icons
-from quodlibet.util.dprint import print_d
+from quodlibet.util.dprint import print_d, print_w
 from quodlibet.util.picklehelper import pickle_load, pickle_dump, PickleError
-from quodlibet.compat import urlencode
 from quodlibet.util.urllib import urlopen, UrllibError
 from quodlibet.errorreport import errorhook
 
-
 SERVICES = {
     'Last.fm': 'http://post.audioscrobbler.com/',
     'Libre.fm': 'http://turtle.libre.fm/'
@@ -46,7 +45,6 @@ DEFAULT_SERVICE = 'Last.fm'
 DEFAULT_TITLEPAT = '<title><version| (<version>)>'
 DEFAULT_ARTISTPAT = '<artist|<artist>|<composer|<composer>|<performer>>>'
 
-
 plugin_config = PluginConfig("scrobbler")
 defaults = plugin_config.defaults
 defaults.set("service", DEFAULT_SERVICE)
@@ -80,7 +78,7 @@ def config_get_artist_pattern():
     return plugin_config.get('artistpat') or DEFAULT_ARTISTPAT
 
 
-class QLSubmitQueue(object):
+class QLSubmitQueue:
     """Manages the submit queue for scrobbles. Works independently of the
     QLScrobbler plugin being enabled; other plugins may use submit() to queue
     songs for scrobbling.
@@ -89,11 +87,11 @@ class QLSubmitQueue(object):
     CLIENT = "qlb"
     CLIENT_VERSION = const.VERSION
     PROTOCOL_VERSION = "1.2"
-    DUMP = os.path.join(quodlibet.get_user_dir(), "scrobbler_cache_v2")
+    SCROBBLER_CACHE_FILE = os.path.join(quodlibet.get_user_dir(), "scrobbler_cache_v2")
 
     # These objects are shared across instances, to allow other plugins to
     # queue scrobbles in future versions of QL
-    queue = []
+    queue: List[Dict[str, str]] = []
     changed_event = threading.Event()
 
     def set_nowplaying(self, song):
@@ -141,6 +139,11 @@ class QLSubmitQueue(object):
         self.nowplaying_song = None
         self.nowplaying_sent = False
         self.sessionid = None
+        self.handshake_event = None
+        self.handshake_delay = 1.0
+        self.failures = 0
+        self.handshake_sent = 0
+        self.session_id, self.nowplaying_url, self.submit_url = None, None, None
 
         self.broken = False
 
@@ -150,10 +153,13 @@ class QLSubmitQueue(object):
         self.titlepat = Pattern(config_get_title_pattern())
         self.artpat = Pattern(config_get_artist_pattern())
 
+        self._load_queue()
+
+    def _load_queue(self):
         try:
-            with open(self.DUMP, 'rb') as disk_queue_file:
+            with open(self.SCROBBLER_CACHE_FILE, 'rb') as disk_queue_file:
                 disk_queue = pickle_load(disk_queue_file)
-            os.unlink(self.DUMP)
+            os.unlink(self.SCROBBLER_CACHE_FILE)
             self.queue += disk_queue
         except (EnvironmentError, PickleError):
             pass
@@ -161,11 +167,12 @@ class QLSubmitQueue(object):
     @classmethod
     def dump_queue(klass):
         if klass.queue:
+            print_d(f"Saving scrobble queue to {klass.SCROBBLER_CACHE_FILE}")
             try:
-                with open(klass.DUMP, 'wb') as disk_queue_file:
+                with open(klass.SCROBBLER_CACHE_FILE, 'wb') as disk_queue_file:
                     pickle_dump(klass.queue, disk_queue_file)
-            except (EnvironmentError, PickleError):
-                pass
+            except (EnvironmentError, PickleError) as e:
+                print_w(f"Couldn't persist scrobble queue ({e})")
 
     def _check_config(self):
         user = plugin_config.get('username')
@@ -174,11 +181,10 @@ class QLSubmitQueue(object):
         if not user or not passw or not url:
             if self.queue and not self.broken:
                 self.quick_dialog(_("Please visit the Plugins window to set "
-                              "QLScrobbler up. Until then, songs will not be "
-                              "submitted."), Gtk.MessageType.INFO)
+                                    "QLScrobbler up. Until then, songs will not be "
+                                    "submitted."), Gtk.MessageType.INFO)
                 self.broken = True
-        elif (self.username, self.password,
-                self.base_url) != (user, passw, url):
+        elif (self.username, self.password, self.base_url) != (user, passw, url):
             self.username, self.password, self.base_url = (user, passw, url)
             self.broken = False
             self.handshake_sent = False
@@ -189,8 +195,8 @@ class QLSubmitQueue(object):
     def changed(self):
         """Signal that settings or queue contents were changed."""
         self._check_config()
-        if not self.broken and not self.offline and (self.queue or
-                (self.nowplaying_song and not self.nowplaying_sent)):
+        pending = self.queue or (self.nowplaying_song and not self.nowplaying_sent)
+        if not self.broken and not self.offline and pending:
             self.changed_event.set()
             return
         self.changed_event.clear()
@@ -240,10 +246,11 @@ class QLSubmitQueue(object):
         stamp = int(time.time())
         auth = md5(self.password.encode("utf-8") +
                    str(stamp).encode("utf-8")).hexdigest()
-        url = "%s/?hs=true&p=%s&c=%s&v=%s&u=%s&a=%s&t=%d" % (
-                    self.base_url, self.PROTOCOL_VERSION, self.CLIENT,
-                    self.CLIENT_VERSION, self.username, auth, stamp)
-        print_d("Sending handshake to service.")
+        qp = dict(hs="true", p=self.PROTOCOL_VERSION, c=self.CLIENT,
+                  v=self.CLIENT_VERSION, u=self.username, a=auth, t=int(stamp))
+        qs = urlencode(qp)
+        url = f"{self.base_url}/?{qs}"
+        print_d(f"Sending handshake to service at {self.base_url}.")
 
         try:
             resp = urlopen(url)
@@ -257,14 +264,13 @@ class QLSubmitQueue(object):
             return False
         except ValueError:
             self.quick_dialog(_("Authentication failed: invalid URL."),
-                Gtk.MessageType.ERROR)
+                              Gtk.MessageType.ERROR)
             self.broken = True
             return False
 
         # check response
         lines = resp.read().decode("utf-8", "ignore").rstrip().split("\n")
         status = lines.pop(0)
-        print_d("Handshake status: %s" % status)
 
         if status == "OK":
             self.session_id, self.nowplaying_url, self.submit_url = lines
@@ -272,10 +278,11 @@ class QLSubmitQueue(object):
             print_d("Session ID: %s, NP URL: %s, Submit URL: %s" % (
                 self.session_id, self.nowplaying_url, self.submit_url))
             return True
-        elif status == "BADAUTH":
+        print_w(f"Bad handshake status: {status}")
+        if status == "BADAUTH":
             self.quick_dialog(_("Authentication failed: Invalid username '%s' "
-                            "or bad password.") % util.escape(self.username),
-                            Gtk.MessageType.ERROR)
+                                "or bad password.") % util.escape(self.username),
+                              Gtk.MessageType.ERROR)
             self.broken = True
         elif status == "BANNED":
             self.quick_dialog(_("Client is banned. Contact the author."),
@@ -283,7 +290,7 @@ class QLSubmitQueue(object):
             self.broken = True
         elif status == "BADTIME":
             self.quick_dialog(_("Wrong system time. Submissions may fail "
-                            "until it is corrected."), Gtk.MessageType.ERROR)
+                                "until it is corrected."), Gtk.MessageType.ERROR)
         else:  # "FAILED"
             self.quick_dialog(util.escape(status), Gtk.MessageType.ERROR)
         self.changed()
@@ -300,7 +307,6 @@ class QLSubmitQueue(object):
         resp_save = resp.read().decode("utf-8", "ignore")
         status = resp_save.rstrip().split("\n")[0]
         print_d("Submission status: %s" % status)
-
         if status == "OK":
             return True
         elif status == "BADSESSION":
@@ -318,8 +324,8 @@ class QLSubmitQueue(object):
             data['o[%d]' % idx] = 'P'
             data['r[%d]' % idx] = ''
 
-        print_d('Submitting song(s): %s' %
-            ('\n\t'.join(['%s - %s' % (s['a'], s['t']) for s in to_submit])))
+        song_info = '\n\t'.join(f"{s['a']} - {s['t']}" for s in to_submit)
+        print_d(f"Submitting song(s): {song_info}")
 
         if self._check_submit(self.submit_url, data):
             del self.queue[:len(to_submit)]
@@ -352,9 +358,13 @@ class QLScrobbler(EventPlugin):
                     "Audioscrobbler services.")
     PLUGIN_ICON = Icons.NETWORK_WORKGROUP
 
+    AUTOSAVE_INTERVAL = 30
+    """How often, in seconds, to save the queue (if any) to disk"""
+
     def __init__(self):
         self.__enabled = False
         self.queue = QLSubmitQueue()
+        self._tid = None
 
         def queue_run():
             try:
@@ -365,6 +375,7 @@ class QLScrobbler(EventPlugin):
         queue_thread = threading.Thread(None, queue_run)
         queue_thread.setDaemon(True)
         queue_thread.start()
+        print_d(f"Started Scrobbler thread {queue_thread}")
 
         self.start_time = 0
         self.unpaused_time = 0
@@ -431,10 +442,18 @@ class QLScrobbler(EventPlugin):
     def enabled(self):
         self.__enabled = True
         print_d("Plugin enabled - accepting new songs.")
+        if not self._tid:
+            print_d("Starting scrobble autosaver")
+            self._tid = GLib.timeout_add_seconds(
+                self.AUTOSAVE_INTERVAL, lambda: QLSubmitQueue.dump_queue() or True)
 
     def disabled(self):
         self.__enabled = False
         print_d("Plugin disabled - not accepting any new songs.")
+        if self._tid:
+            print_d("Stopping scrobble autosaver")
+            GLib.source_remove(self._tid)
+            self._tid = None
         QLSubmitQueue.dump_queue()
 
     def PluginPreferences(self, parent):
@@ -454,7 +473,7 @@ class QLScrobbler(EventPlugin):
             status = queue.send_handshake(show_dialog=True)
             if status:
                 queue.quick_dialog(_("Authentication successful."),
-                    Gtk.MessageType.INFO)
+                                   Gtk.MessageType.INFO)
 
         box = Gtk.VBox(spacing=12)
 
@@ -465,15 +484,14 @@ class QLScrobbler(EventPlugin):
         table.set_row_spacings(6)
 
         labels = []
-        label_names = [_("_Service:"), _("_URL:"), _("User_name:"),
-            _("_Password:")]
+        label_names = [_("_Service:"), _("_URL:"), _("User_name:"), _("_Password:")]
         for idx, name in enumerate(label_names):
             label = Gtk.Label(label=name)
             label.set_alignment(0.0, 0.5)
             label.set_use_underline(True)
             table.attach(label, 0, 1, idx, idx + 1,
                          xoptions=Gtk.AttachOptions.FILL |
-                         Gtk.AttachOptions.SHRINK)
+                                  Gtk.AttachOptions.SHRINK)
             labels.append(label)
 
         row = 0
@@ -534,7 +552,7 @@ class QLScrobbler(EventPlugin):
         table.set_row_spacings(6)
 
         label_names = [_("_Artist pattern:"), _("_Title pattern:"),
-            _("Exclude _filter:")]
+                       _("Exclude _filter:")]
 
         labels = []
         for idx, name in enumerate(label_names):
@@ -543,7 +561,7 @@ class QLScrobbler(EventPlugin):
             label.set_use_underline(True)
             table.attach(label, 0, 1, idx, idx + 1,
                          xoptions=Gtk.AttachOptions.FILL |
-                         Gtk.AttachOptions.SHRINK)
+                                  Gtk.AttachOptions.SHRINK)
             labels.append(label)
 
         row = 0
@@ -553,7 +571,8 @@ class QLScrobbler(EventPlugin):
         entry.connect('changed', changed, 'artistpat')
         table.attach(entry, 1, 2, row, row + 1)
         entry.set_tooltip_text(_("The pattern used to format "
-            "the artist name for submission. Leave blank for default."))
+                                 "the artist name for submission. Leave blank for "
+                                 "default."))
         labels[row].set_mnemonic_widget(entry)
         row += 1
 
@@ -563,7 +582,7 @@ class QLScrobbler(EventPlugin):
         entry.connect('changed', changed, 'titlepat')
         table.attach(entry, 1, 2, row, row + 1)
         entry.set_tooltip_text(_("The pattern used to format "
-            "the title for submission. Leave blank for default."))
+                                 "the title for submission. Leave blank for default."))
         labels[row].set_mnemonic_widget(entry)
         row += 1
 
@@ -571,7 +590,7 @@ class QLScrobbler(EventPlugin):
         entry = ValidatingEntry(Query.validator)
         entry.set_text(plugin_config.get('exclude'))
         entry.set_tooltip_text(
-                _("Songs matching this filter will not be submitted."))
+            _("Songs matching this filter will not be submitted"))
         entry.connect('changed', changed, 'exclude')
         table.attach(entry, 1, 2, row, row + 1)
         labels[row].set_mnemonic_widget(entry)
@@ -579,8 +598,8 @@ class QLScrobbler(EventPlugin):
 
         # offline mode
         offline = plugin_config.ConfigCheckButton(
-                _("_Offline mode (don't submit anything)"),
-                'offline', populate=True)
+            _("_Offline mode (don't submit anything)"),
+            'offline', populate=True)
         table.attach(offline, 0, 2, row, row + 1)
 
         box.pack_start(qltk.Frame(_("Submission"), child=table), True, True, 0)
