@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2009 Joe Wreschnig, Michael Urman, Steven Robertson
 #           2011-2017 Nick Boultbee
 #
@@ -9,7 +8,6 @@
 
 from gi.repository import GObject, Gtk
 
-from quodlibet.browsers.playlists import PlaylistsBrowser
 from quodlibet.browsers.playlists.menu import PlaylistMenu
 
 from quodlibet import _
@@ -22,16 +20,17 @@ from quodlibet.qltk import Icons
 from quodlibet.qltk.browser import LibraryBrowser
 from quodlibet.qltk.information import Information
 from quodlibet.qltk.properties import SongProperties
+from quodlibet.qltk.util import GSignals
 
 
 class IndicatorMenu(Gtk.Menu):
 
-    __gsignals__ = {
+    __gsignals__: GSignals = {
         'action-item-changed': (GObject.SignalFlags.RUN_LAST, None, tuple()),
     }
 
     def __init__(self, app, add_show_item=False):
-        super(IndicatorMenu, self).__init__()
+        super().__init__()
 
         self._app = app
         player = app.player
@@ -193,13 +192,10 @@ class IndicatorMenu(Gtk.Menu):
 
     def _new_playlist_submenu_for(self, song):
         submenu = self._playlists_item.get_submenu()
+        pl_lib = self._app.library and self._app.library.playlists
         if submenu:
             submenu.destroy()
-        playlist_menu = PlaylistMenu([song], PlaylistsBrowser.playlists())
-
-        def on_new(widget, playlist):
-            PlaylistsBrowser.changed(playlist)
-        playlist_menu.connect('new', on_new)
+        playlist_menu = PlaylistMenu([song], pl_lib)
         self._playlists_item.set_submenu(playlist_menu)
         self._playlists_item.set_sensitive(bool(song) and song.can_add)
         self._playlists_item.show_all()
