@@ -13,10 +13,13 @@ class ReverseOrder(ShufflePlugin, OrderInOrder):
     PLUGIN_ID = "reverse"
     PLUGIN_NAME = _("Reverse")
     PLUGIN_ICON = Icons.MEDIA_SKIP_BACKWARD
-    PLUGIN_DESC = _("Reverses the play order of songs.")
+    PLUGIN_DESC = _("Adds a shuffle mode "
+                    "that reverses the play order of songs.")
+    display_name = _("Reverse")
+    accelerated_name = _("Re_verse")
 
     def previous(self, playlist, iter):
-        return super(ReverseOrder, self).next(playlist, iter)
+        return super().next(playlist, iter)
 
     def next(self, playlist, iter):
-        return super(ReverseOrder, self).previous(playlist, iter)
+        return super().previous(playlist, iter)
