@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -7,23 +6,11 @@
 # (at your option) any later version.
 
 try:
-    # https://github.com/alastair/python-musicbrainzngs/issues/157
-    # Work around warnings getting enabled in musicbrainzngs
-    import warnings
-    f = list(warnings.filters)
     import musicbrainzngs
-    warnings.filters[:] = f
 except ImportError:
     from quodlibet import plugins
     raise plugins.MissingModulePluginException("musicbrainzngs")
 
-
-# musicbrainzngs.get_url_by_id was added in version 0.5
-if not hasattr(musicbrainzngs, "get_url_by_id"):
-    from quodlibet import plugins
-    raise plugins.MissingModulePluginException("musicbrainzngs >= 0.5")
-
-
 from quodlibet import app
 from quodlibet import const
 from quodlibet import util
@@ -57,7 +44,7 @@ def _get_release(release_id):
     )["release"]
 
 
-class Artist(object):
+class Artist:
 
     def __init__(self, name, sort_name, id_):
         self.name = name
@@ -83,7 +70,7 @@ class Artist(object):
         return artists
 
 
-class ReleaseTrack(object):
+class ReleaseTrack:
     """Part of a Release, combines per track and per medium data"""
 
     def __init__(self, mbtrack, discnumber, track_count, disctitle):
@@ -111,7 +98,7 @@ class ReleaseTrack(object):
         return self._mbtrack["position"]
 
 
-class Release(object):
+class Release:
 
     def __init__(self, mbrelease):
         self._mbrelease = mbrelease
@@ -206,7 +193,7 @@ class Release(object):
         """Returns a new Release instance containing more info
         or raises MusicBrainzError
 
-        This method is blocking..
+        This method is blocking...
         """
 
         return Release(_get_release(self.id))
