@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
-# Copyright 2012-2016 Nick Boultbee
+# Copyright 2012-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk
 
@@ -11,9 +11,10 @@ import os
 
 from quodlibet.util.songwrapper import SongWrapper
 
-from quodlibet.qltk.songsmenu import ConfirmMultiSongInvoke
+from quodlibet.qltk.songsmenu import confirm_multi_song_invoke
 
 import quodlibet
+from quodlibet import _
 from quodlibet import qltk
 from quodlibet import util
 from quodlibet.pattern import Pattern
@@ -64,16 +65,20 @@ class Command(JSONObject):
         "max_args": Field(_("max args"),
                           _("The maximum number of argument to pass to the "
                             "command at one time (like xargs)")),
+
+        "reverse": Field(_("reverse"),
+                          _("If set, the argument list will be reversed")),
     }
 
     def __init__(self, name=None, command=None, pattern="<~filename>",
-                 unique=False, playlist=False, parameter=None,
-                 max_args=10000, warn_threshold=50):
+                 unique=False, parameter=None, max_args=10000, reverse=False,
+                 warn_threshold=50):
         JSONObject.__init__(self, name)
         self.command = str(command or "")
         self.pattern = str(pattern)
         self.unique = bool(unique)
         self.max_args = max_args
+        self.reverse = bool(reverse)
         self.parameter = str(parameter or "")
         self.__pat = Pattern(self.pattern)
         self.warn_threshold = warn_threshold
@@ -93,8 +98,9 @@ class Command(JSONObject):
         if playlist_name:
             print_d("Playlist command for %s" % playlist_name)
             template_vars["PLAYLIST"] = playlist_name
-        self.command = self.command.format(**template_vars)
-        print_d("Actual command=%s" % self.command)
+
+        actual_command = self.command.format(**template_vars)
+        print_d("Actual command=%s" % actual_command)
         for i, song in enumerate(songs):
             wrapped = SongWrapper(song)
             if playlist_name:
@@ -111,19 +117,28 @@ class Command(JSONObject):
             elif arg not in args:
                 args.append(arg)
         max = int((self.max_args or 10000))
-        com_words = self.command.split(" ")
+        com_words = actual_command.split(" ")
+
+        if self.reverse:
+            args.reverse()
+
         while args:
             print_d("Running %s with %d substituted arg(s) (of %d%s total)..."
-                    % (self.command, min(max, len(args)), len(args),
+                    % (actual_command, min(max, len(args)), len(args),
                        " unique" if self.unique else ""))
             util.spawn(com_words + args[:max])
             args = args[max:]
 
+    @property
+    def playlists_only(self):
+        return ("~playlistname" in self.pattern
+                or "playlistindex" in self.pattern)
+
     def __str__(self):
-        return 'Command: "{command} {pattern}"'.format(**dict(self.data))
+        return "{command} {pattern}".format(**dict(self.data))
 
 
-class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
+class CustomCommands(PlaylistPlugin, SongsMenuPlugin, PluginConfigMixin):
 
     PLUGIN_ICON = Icons.APPLICATION_UTILITIES
     PLUGIN_ID = "CustomCommands"
@@ -138,10 +153,18 @@ class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
         Command("Browse folders (Thunar)", "thunar", "<~dirname>", unique=True,
                 max_args=50, warn_threshold=20),
 
-        Command(name="Flash notification",
+        Command("Burn using K3b", "k3b --audiocd"),
+
+        Command("Burn using Brasero", "brasero --audio"),
+
+        Command("Burn using Xfburn", "xfburn --audio-composition", reverse=True),
+
+        Command("Flash notification",
                 command="notify-send"
                     " -t 2000"
-                    " -i /usr/share/icons/hicolor/scalable/apps/quodlibet.svg",
+                    " -i "
+                        "/usr/share/icons/hicolor/scalable/apps/"
+                        "io.github.quodlibet.QuodLibet.svg",
                 pattern="<~rating> \"<title><version| (<version>)>\""
                         "<~people| by <~people>>"
                     "<album|, from <album><discnumber| : disk <discnumber>>"
@@ -149,7 +172,7 @@ class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
                 max_args=1,
                 warn_threshold=10),
 
-        Command(name="Output playlist to stdout",
+        Command("Output playlist to stdout",
                 command="echo -e",
                 pattern="<~playlistname>: <~playlistindex>. "
                         " <~artist~title>\\\\n",
@@ -157,26 +180,37 @@ class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
 
         Command("Fix MP3 VBR with mp3val", "mp3val -f", unique=True,
                 max_args=1),
+
+        Command("Record Stream",
+                command="x-terminal-emulator -e wget -P $HOME",
+                pattern="<~filename>",
+                max_args=1)
     ]
+
     COMS_FILE = os.path.join(
         quodlibet.get_user_dir(), 'lists', 'customcommands.json')
 
+    _commands = None
+    """Commands known to the class"""
+
     def __set_pat(self, name):
         self.com_index = name
 
     def get_data(self, key):
         """Gets the pattern for a given key"""
         try:
-            return self.commands[key]
+            return self.all_commands()[key]
         except (KeyError, TypeError):
             print_d("Invalid key %s" % key)
             return None
 
     @classmethod
     def edit_patterns(cls, button):
-        cls.commands = cls._get_saved_searches()
-        win = JSONBasedEditor(Command, cls.commands, filename=cls.COMS_FILE,
+        win = JSONBasedEditor(Command, cls.all_commands(),
+                              filename=cls.COMS_FILE,
                               title=_("Edit Custom Commands"))
+        # Cache busting
+        cls._commands = None
         win.show()
 
     @classmethod
@@ -184,21 +218,25 @@ class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
         hb = Gtk.HBox(spacing=3)
         hb.set_border_width(0)
 
-        button = qltk.Button(_("Edit Custom Commands") + "...", Icons.EDIT)
-        button.set_tooltip_markup(util.escape(_("Supports QL patterns\neg "
-                                                "<tt><~artist~title></tt>")))
+        button = qltk.Button(_("Edit Custom Commands") + "…", Icons.EDIT)
         button.connect("clicked", cls.edit_patterns)
         hb.pack_start(button, True, True, 0)
         hb.show_all()
         return hb
 
     @classmethod
-    def _get_saved_searches(cls):
+    def all_commands(cls):
+        if cls._commands is None:
+            cls._commands = cls._get_saved_commands()
+        return cls._commands
+
+    @classmethod
+    def _get_saved_commands(cls):
         filename = cls.COMS_FILE
         print_d("Loading saved commands from '%s'..." % filename)
         coms = None
         try:
-            with open(filename) as f:
+            with open(filename, "r", encoding="utf-8") as f:
                 coms = JSONObjectDict.from_json(Command, f.read())
         except (IOError, ValueError) as e:
             print_w("Couldn't parse saved commands (%s)" % e)
@@ -206,22 +244,23 @@ class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
         # Failing all else...
         if not coms:
             print_d("No commands found in %s. Using defaults." % filename)
-            coms = dict([(c.name, c) for c in cls.DEFAULT_COMS])
+            coms = {c.name: c for c in cls.DEFAULT_COMS}
         print_d("Loaded commands: %s" % coms.keys())
         return coms
 
     def __init__(self, *args, **kwargs):
-        super(CustomCommands, self).__init__(*args, **kwargs)
+        super().__init__(**kwargs)
+        pl_mode = hasattr(self, '_playlists') and bool(len(self._playlists))
         self.com_index = None
         self.unique_only = False
-        self.commands = {}
         submenu = Gtk.Menu()
-        self.commands = self._get_saved_searches()
-        for (name, c) in self.commands.items():
+        for name, c in self.all_commands().items():
             item = Gtk.MenuItem(label=name)
             connect_obj(item, 'activate', self.__set_pat, name)
+            if pl_mode and not c.playlists_only:
+                continue
+            item.set_sensitive(c.playlists_only == pl_mode)
             submenu.append(item)
-            # Add link to editor
 
         self.add_edit_item(submenu)
         if submenu.get_children():
@@ -231,7 +270,7 @@ class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
 
     @classmethod
     def add_edit_item(cls, submenu):
-        config = Gtk.MenuItem(label=_("Edit Custom Commands") + "...")
+        config = Gtk.MenuItem(label=_("Edit Custom Commands") + "…")
         connect_obj(config, 'activate', cls.edit_patterns, config)
         config.set_sensitive(not JSONBasedEditor.is_not_unique())
         submenu.append(SeparatorMenuItem())
@@ -249,7 +288,7 @@ class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
         if self.com_index:
             com = self.get_data(self.com_index)
             if len(songs) > com.warn_threshold:
-                if not ConfirmMultiSongInvoke.confirm(
+                if not confirm_multi_song_invoke(
                         self, com.name, len(songs)):
                     print_d("User decided not to run on %d songs" % len(songs))
                     return
@@ -262,6 +301,6 @@ class CustomCommands(SongsMenuPlugin, PlaylistPlugin, PluginConfigMixin):
                 print_exc()
                 ErrorMessage(
                     self.plugin_window,
-                    _("Unable to run custom command %s" %
-                      util.escape(self.com_index)),
+                    _("Unable to run custom command %s") %
+                    util.escape(self.com_index),
                     util.escape(str(err))).run()
