@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter
-#     2013, 2016 Nick Boultbee
+#      2013-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -9,31 +8,32 @@
 
 from gi.repository import Gtk
 
-from quodlibet import _
+from quodlibet import _, print_d, ngettext
 from quodlibet import app
 from quodlibet import util
 from quodlibet.plugins.songshelpers import any_song, has_writable_image
+from quodlibet.qltk.chooser import _get_chooser
 from quodlibet.qltk.x import MenuItem
 from quodlibet.qltk import Icons
 from quodlibet.qltk.wlw import WritingWindow
-from quodlibet.formats import EmbeddedImage, AudioFileError
+from quodlibet.qltk._editutils import WriteFailedError
+from quodlibet.formats import EmbeddedImage, AudioFileError, AudioFile
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 
 
 class EditEmbedded(SongsMenuPlugin):
     PLUGIN_ID = "embedded_edit"
     PLUGIN_NAME = _("Edit Embedded Images")
-    PLUGIN_DESC = _("Removes or replaces embedded images.")
+    PLUGIN_DESC = _("Adds, removes or replaces embedded images.")
     PLUGIN_ICON = Icons.INSERT_IMAGE
 
     plugin_handles = any_song(has_writable_image)
     """if any song supports editing, we are active"""
 
     def __init__(self, songs, *args, **kwargs):
-        super(EditEmbedded, self).__init__(songs, *args, **kwargs)
+        super().__init__(songs, *args, **kwargs)
         self.__menu = Gtk.Menu()
-        self.__menu.connect('map', self.__map, songs)
-        self.__menu.connect('unmap', self.__unmap)
+        self._init_submenu_items(self.__menu, songs)
         self.set_submenu(self.__menu)
 
     def __remove_images(self, menu_item, songs):
@@ -46,6 +46,7 @@ class EditEmbedded(SongsMenuPlugin):
                     song.clear_images()
                 except AudioFileError:
                     util.print_exc()
+                    WriteFailedError(win, song).run()
 
             if win.step():
                 break
@@ -64,10 +65,35 @@ class EditEmbedded(SongsMenuPlugin):
                     path = fileobj.name
                     image = EmbeddedImage.from_path(path)
                     if image:
-                        try:
-                            song.set_image(image)
-                        except AudioFileError:
-                            util.print_exc()
+                        _set_image_or_throw(image, song, win)
+            if win.step():
+                break
+
+        win.destroy()
+        self.plugin_finish()
+
+    def __choose_image(self, menu_item, songs):
+
+        dialog = _get_chooser(_('_Embed'), _('_Cancel'))
+        total = len(songs)
+        msg = ngettext("Choose image to embed in the %d track",
+                       "Choose image to embed in %d tracks", total) % total
+        dialog.set_title(msg)
+        response = dialog.run()
+        path = dialog.get_filename()
+        dialog.destroy()
+        if response != Gtk.ResponseType.ACCEPT:
+            print_d("User cancelled image embedding")
+            return
+
+        win = WritingWindow(self.plugin_window, len(songs))
+        win.show()
+
+        for song in songs:
+            if song.can_change_images:
+                image = EmbeddedImage.from_path(path)
+                if image:
+                    _set_image_or_throw(image, song, win)
 
             if win.step():
                 break
@@ -75,20 +101,30 @@ class EditEmbedded(SongsMenuPlugin):
         win.destroy()
         self.plugin_finish()
 
-    def __map(self, menu, songs):
-        remove_item = MenuItem(_("_Remove all Images"), "edit-delete")
+    def _init_submenu_items(self, menu, songs):
+        remove_item = MenuItem(_("_Remove all Images"), Icons.EDIT_DELETE)
         remove_item.connect('activate', self.__remove_images, songs)
+        remove_item.set_sensitive(any(song.has_images for song in songs))
         menu.append(remove_item)
 
-        set_item = MenuItem(_("_Embed Current Image"), "edit-paste")
+        set_item = MenuItem(_("_Embed Current Image"), Icons.EDIT_PASTE)
         set_item.connect('activate', self.__set_image, songs)
         menu.append(set_item)
 
-        menu.show_all()
+        set_item = MenuItem(_("_Choose Image…"), Icons.DOCUMENT_OPEN)
+        set_item.connect('activate', self.__choose_image, songs)
+        menu.append(set_item)
 
-    def __unmap(self, menu):
-        for child in self.__menu.get_children():
-            self.__menu.remove(child)
+        menu.show_all()
+        return menu
 
     def plugin_songs(self, songs):
         return True
+
+
+def _set_image_or_throw(image: EmbeddedImage, song: AudioFile, win: Gtk.Window) -> None:
+    try:
+        song.set_image(image)
+    except AudioFileError:
+        util.print_exc()
+        WriteFailedError(win, song).run()
