@@ -1,20 +1,20 @@
-# -*- coding: utf-8 -*-
 #    ReplayGain Album Analysis using gstreamer rganalysis element
 #    Copyright (C) 2005,2007,2009  Michael Urman
-#                  2012,2014,2016  Nick Boultbee
+#                       2012-2021  Nick Boultbee
 #                            2013  Christoph Reiter
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of version 2 of the GNU General Public License as
-#    published by the Free Software Foundation.
-#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk
 from gi.repository import GObject
 from gi.repository import Pango
 from gi.repository import Gst
 from gi.repository import GLib
-from quodlibet import print_d
+
+from quodlibet import print_d, ngettext, C_, _, util
 from quodlibet.plugins import PluginConfigMixin
 
 from quodlibet.browsers.collection.models import EMPTY
@@ -24,7 +24,8 @@ from quodlibet.qltk.x import Frame
 from quodlibet.qltk import Icons, Dialog
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.plugins.songshelpers import is_writable, is_finite, each_song
-from quodlibet.util import cached_property
+from quodlibet.util import cached_property, print_w, print_e, format_int_locale
+from quodlibet.util.path import uri2gsturi
 
 __all__ = ['ReplayGain']
 
@@ -41,14 +42,14 @@ def get_num_threads():
     return threads
 
 
-class UpdateMode(object):
+class UpdateMode:
     """Enum-like class for update strategies"""
     ALWAYS = "always"
     ALBUM_MISSING = "album_tags_missing"
     ANY_MISSING = "any_tags_missing"
 
 
-class RGAlbum(object):
+class RGAlbum:
     def __init__(self, rg_songs, process_mode):
         self.songs = rg_songs
         self.gain = None
@@ -120,7 +121,7 @@ class RGAlbum(object):
             return True
 
 
-class RGSong(object):
+class RGSong:
     def __init__(self, song):
         self.song = song
         self.error = False
@@ -152,6 +153,13 @@ class RGSong(object):
         write_to_song('replaygain_album_gain', '%.2f dB', album_gain)
         write_to_song('replaygain_album_peak', '%.4f', album_peak)
 
+        # bs1770gain writes those and since we still do old replaygain
+        # just delete them so players use the defaults.
+        song.pop("replaygain_reference_loudness", None)
+        song.pop("replaygain_algorithm", None)
+        song.pop("replaygain_album_range", None)
+        song.pop("replaygain_track_range", None)
+
     @property
     def title(self):
         return self.song('~tracknumber~title~version')
@@ -160,6 +168,10 @@ class RGSong(object):
     def filename(self):
         return self.song("~filename")
 
+    @property
+    def uri(self):
+        return self.song("~uri")
+
     @property
     def length(self):
         return self.song("~#length")
@@ -203,7 +215,6 @@ class RGSong(object):
 
 
 class ReplayGainPipeline(GObject.Object):
-
     __gsignals__ = {
         # done(self, album)
         'done': (GObject.SignalFlags.RUN_LAST, None, (object,)),
@@ -213,35 +224,22 @@ class ReplayGainPipeline(GObject.Object):
     }
 
     def __init__(self):
-        super(ReplayGainPipeline, self).__init__()
+        super().__init__()
 
         self._current = None
         self._setup_pipe()
 
     def _setup_pipe(self):
         # gst pipeline for replay gain analysis:
-        # filesrc!decodebin!audioconvert!audioresample!rganalysis!fakesink
+        # uridecodebin!audioconvert!audioresample!rganalysis!fakesink
         self.pipe = Gst.Pipeline()
-        self.filesrc = Gst.ElementFactory.make("filesrc", "source")
-        self.pipe.add(self.filesrc)
-
-        self.decode = Gst.ElementFactory.make("decodebin", "decode")
+        self.decode = Gst.ElementFactory.make("uridecodebin", "decode")
 
         def new_decoded_pad(dbin, pad):
             pad.link(self.convert.get_static_pad("sink"))
 
-        def sort_decoders(decode, pad, caps, factories):
-            def set_prio(x):
-                i, f = x
-                i = {"mad": -1, "mpg123audiodec": -2}.get(f.get_name(), i)
-                return (i, f)
-            return zip(*sorted(map(set_prio, enumerate(factories))))[1]
-
-        self.decode.connect("autoplug-sort", sort_decoders)
-
         self.decode.connect("pad-added", new_decoded_pad)
         self.pipe.add(self.decode)
-        self.filesrc.link(self.decode)
 
         self.convert = Gst.ElementFactory.make("audioconvert", "convert")
         self.pipe.add(self.convert)
@@ -310,7 +308,7 @@ class ReplayGainPipeline(GObject.Object):
             self.pipe.set_state(Gst.State.NULL)
 
         self._current = self._songs.pop(0)
-        self.filesrc.set_property("location", self._current.filename)
+        self.decode.set_property("uri", uri2gsturi(self._current.uri))
         if not first:
             # flush, so the element takes new data after EOS
             pad = self.analysis.get_static_pad("src")
@@ -349,7 +347,7 @@ class ReplayGainPipeline(GObject.Object):
 class RGDialog(Dialog):
 
     def __init__(self, albums, parent, process_mode):
-        super(RGDialog, self).__init__(
+        super().__init__(
             title=_('ReplayGain Analyzer'), parent=parent)
 
         self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
@@ -392,7 +390,8 @@ class RGDialog(Dialog):
             cell.set_property('text', item.title)
             cell.set_sensitive(model[iter_][1])
 
-        column = Gtk.TreeViewColumn(_("Track"))
+        # Translators: Combined track number/title column heading
+        column = Gtk.TreeViewColumn(C_("track/title", "Track"))
         column.set_expand(True)
         column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         track_render = Gtk.CellRendererText()
@@ -451,19 +450,19 @@ class RGDialog(Dialog):
         self.__fill_view(view, albums)
         num_to_process = sum(int(rga.should_process) for rga in self._todo)
         template = ngettext(
-            "There is <b>%(to-process)d</b> album to update (of %(all)d)",
-            "There are <b>%(to-process)d</b> albums to update (of %(all)d)",
+            "There is %(to-process)s album to update (of %(all)s)",
+            "There are %(to-process)s albums to update (of %(all)s)",
             num_to_process)
         info.set_markup(template % {
-            "to-process": num_to_process,
-            "all": len(self._todo),
+            "to-process": util.bold(format_int_locale(num_to_process)),
+            "all": util.bold(format_int_locale(len(self._todo))),
         })
         self.connect("destroy", self.__destroy)
         self.connect('response', self.__response)
 
     def create_pipelines(self):
         # create as many pipelines as threads
-        self.pipes = [ReplayGainPipeline() for _ in xrange(get_num_threads())]
+        self.pipes = [ReplayGainPipeline() for _ in range(get_num_threads())]
 
     def __fill_view(self, view, albums):
         self._todo = [RGAlbum.from_songs(a, self.process_mode) for a in albums]
@@ -542,8 +541,9 @@ class RGDialog(Dialog):
 
     def __done(self, pipeline, album):
         self._done.append(album)
-        if self._todo:
-            pipeline.start(self._todo.pop(0))
+        next_album = self.get_next_album()
+        if next_album:
+            pipeline.start(next_album)
         self.__update_view_for(album)
 
     def __update_view_for(self, album):
@@ -567,8 +567,10 @@ class RGDialog(Dialog):
 class ReplayGain(SongsMenuPlugin, PluginConfigMixin):
     PLUGIN_ID = 'ReplayGain'
     PLUGIN_NAME = _('Replay Gain')
-    PLUGIN_DESC = _('Analyzes and updates ReplayGain information, '
-                    'using GStreamer. Results are grouped by album.')
+    PLUGIN_DESC_MARKUP = (
+        _('Analyzes and updates <a href=\"%(rg_link)s\">ReplayGain</a> information, '
+          'using GStreamer. Results are grouped by album.')
+        % {"rg_link": _("https://en.wikipedia.org/wiki/ReplayGain")})
     PLUGIN_ICON = Icons.MULTIMEDIA_VOLUME_CONTROL
     CONFIG_SECTION = 'replaygain'
 
@@ -592,19 +594,20 @@ class ReplayGain(SongsMenuPlugin, PluginConfigMixin):
 
         # Tabulate all settings for neatness
         table = Gtk.Table(n_rows=1, n_columns=2)
+        table.props.expand = False
         table.set_col_spacings(6)
         table.set_row_spacings(6)
         rows = []
 
         def process_option_changed(combo):
-            #xcode = combo.get_child().get_text()
+            # xcode = combo.get_child().get_text()
             model = combo.get_model()
             lbl, value = model[combo.get_active()]
             cls.config_set("process_if", value)
 
         def create_model():
             model = Gtk.ListStore(str, str)
-            model.append(["<b>%s</b>" % _("always"), UpdateMode.ALWAYS])
+            model.append([util.bold(_("always")), UpdateMode.ALWAYS])
             model.append([_("if <b>any</b> RG tags are missing"),
                           UpdateMode.ANY_MISSING])
             model.append([_("if <b>album</b> RG tags are missing"),
