@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2006 Joe Wreschnig, Michael Urman, Niklas Janlert
 #
 # This program is free software; you can redistribute it and/or modify
@@ -24,6 +23,7 @@ class MP3File(ID3File):
         self["~#bitrate"] = int(info.bitrate / 1000)
         self["~format"] = u"MP%d" % info.layer
         self["~#channels"] = info.channels
+        self["~#samplerate"] = info.sample_rate
 
         encoder, brm = info.encoder_info, info.bitrate_mode
         brm = {1: u"CBR", 2: u"VBR", 3: u"ABR"}.get(brm, u"")
