@@ -3,14 +3,19 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+import os
 import time
 from pathlib import Path
-from typing import Generator, Set, Iterable, Optional
+from typing import Generator, Set, Iterable, Optional, Dict, Tuple, Union
+
+from gi.repository import Gio, GLib, GObject
 
 from quodlibet import print_d, print_w, _, formats
 from quodlibet.formats import AudioFileError, AudioFile
 from quodlibet.library.base import iter_paths, Library, PicklingMixin
 from quodlibet.qltk.notif import Task
+from quodlibet.util import copool, print_exc
+from quodlibet.util.library import get_exclude_dirs
 from quodlibet.util.path import ismount, unexpand, normalize_path
 from senf import fsn2text, fsnative
 
@@ -185,22 +190,20 @@ class FileLibrary(Library[fsnative, AudioFile], PicklingMixin):
         for value in self.scan(paths, exclude, cofuncid):
             yield value
 
-    def add_filename(self, filename, add=True):
+    def add_filename(self,
+                     filename: Union[str, Path],
+                     add: bool = True) -> Optional[AudioFile]:
         """Add a file based on its filename.
-
         Subclasses must override this to open the file correctly.
+
+        :return: the audio file if added (or None)
         """
+        pass
 
-        raise NotImplementedError
-
-    def contains_filename(self, filename):
-        """Returns if a song for the passed filename is in the library.
-
-        Returns:
-            bool
-        """
-
-        raise NotImplementedError
+    def contains_filename(self, filename) -> bool:
+        """Returns if a song for the passed filename is in the library. """
+        key = normalize_path(filename, True)
+        return key in self._contents
 
     def scan(self, paths: Iterable[fsnative],
              exclude: Optional[Iterable[fsnative]] = None,
@@ -323,8 +326,10 @@ class FileLibrary(Library[fsnative, AudioFile], PicklingMixin):
 
         self._masked.pop(mount_point, {})
 
-    def move_root(self, old_root: str, new_root: fsnative) \
-        -> Generator[None, None, None]:
+    def move_root(self,
+                  old_root: str,
+                  new_root: fsnative,
+                  write_files: bool = True) -> Generator[None, None, None]:
         """
         Move the root for all songs in a given (scan) directory.
 
@@ -335,13 +340,15 @@ class FileLibrary(Library[fsnative, AudioFile], PicklingMixin):
           4. Move audio files: old_root -> new_path
 
         """
+        # TODO: only move primary library
         old_path = Path(normalize_path(old_root, canonicalise=True)).expanduser()
         new_path = Path(normalize_path(new_root)).expanduser()
         if not old_path.is_dir():
-            raise ValueError(f"Source {old_path!r} is not a directory")
+            print_w(f"Source dir {str(old_path)!r} doesn't exist, assuming that's OK",
+                    self._name)
         if not new_path.is_dir():
             raise ValueError(f"Destination {new_path!r} is not a directory")
-        print_d(f"{self._name}: checking entire library for {old_path!r}")
+        print_d(f"Checking entire library for {str(old_path)!r}", self._name)
         missing: Set[AudioFile] = set()
         changed = set()
         total = len(self)
@@ -358,24 +365,27 @@ class FileLibrary(Library[fsnative, AudioFile], PicklingMixin):
                     new_key = key.replace(str(old_path), str(new_path), 1)
                     new_key = normalize_path(new_key, canonicalise=False)
                     if new_key == key:
-                        print_w(f"Substitution failed for {key!r}")
+                        print_w(f"Substitution failed for {key!r}", self._name)
                     # We need to update ~filename and ~mountpoint
                     song.sanitize()
-                    song.write()
+                    if write_files:
+                        song.write()
                     if self.move_song(song, new_key):
                         changed.add(song)
                     else:
                         missing.add(song)
                 elif not (i % 1000):
-                    print_d(f"Not moved, for example: {key!r}")
+                    print_d(f"Not moved, for example: {key!r}", self._name)
                 if not i % 100:
                     yield
             self.changed(changed)
             if missing:
-                print_w(f"Couldn't find {len(list(missing))} files: {missing}")
+                print_w(f"Couldn't find {len(list(missing))} files: {missing}",
+                        self._name)
         yield
         self.save()
-        print_d(f"Done moving to {new_path!r}.")
+        print_d(f"Done moving {len(changed)} track(s) (of {total}) "
+                f"to {str(new_path)!r}.", self._name)
 
     def move_song(self, song: AudioFile, new_path: fsnative) -> bool:
         """Updates the location of a song, without touching the file.
@@ -384,7 +394,7 @@ class FileLibrary(Library[fsnative, AudioFile], PicklingMixin):
         """
         existed = True
         key = song.key
-        print_d(f"Moving {key!r} -> {new_path!r}")
+        print_d(f"Moving {key!r} -> {new_path!r}", self._name)
         try:
             del self._contents[key]  # type: ignore
         except KeyError:
@@ -400,7 +410,7 @@ class FileLibrary(Library[fsnative, AudioFile], PicklingMixin):
                      for root in old_roots]
         total = len(self)
         removed = set()
-        print_d(f"Removing library roots {old_roots} from {self._name} library")
+        print_d(f"Removing library roots {old_roots}", self._name)
         yield
         with Task(_("Library"), _("Removing library files")) as task:
             for i, song in enumerate(list(self.values())):
@@ -414,4 +424,175 @@ class FileLibrary(Library[fsnative, AudioFile], PicklingMixin):
         if removed:
             self.remove(removed)
         else:
-            print_d(f"No tracks in {old_roots} to remove from {self._name}")
+            print_d(f"No tracks in {old_roots} to remove", self._name)
+
+
+Event = Gio.FileMonitorEvent
+
+
+class WatchedFileLibraryMixin(FileLibrary):
+    """A File Library that sets up monitors on directories at refresh
+    and handles changes sensibly"""
+
+    def __init__(self, name=None):
+        super().__init__(name)
+        self._monitors: Dict[Path, Tuple[GObject.GObject, int]] = {}
+        print_d(f"Initialised {self!r}")
+
+    def monitor_dir(self, path: Path) -> None:
+        """Monitors a single directory"""
+
+        # Only add one monitor per absolute path...
+        if path not in self._monitors:
+            f = Gio.File.new_for_path(str(path))
+            try:
+                monitor = f.monitor_directory(Gio.FileMonitorFlags.WATCH_MOVES, None)
+            except GLib.GError as e:
+                print_w(f"Couldn't watch {path} ({e})", self._name)
+                monitor = None
+            if not monitor:
+                return
+            handler_id = monitor.connect("changed", self.__file_changed)
+            # Don't destroy references - http://stackoverflow.com/q/4535227
+            self._monitors[path] = (monitor, handler_id)
+            print_d(f"Monitoring {path!s}", self._name)
+
+    def __file_changed(self, _monitor, main_file: Gio.File,
+                       other_file: Optional[Gio.File],
+                       event: Gio.FileMonitorEvent) -> None:
+        if event == Event.CHANGES_DONE_HINT:
+            # This seems to work fine on most Linux, but not on Windows / macOS
+            # Or at least, not in CI anyway.
+            # So shortcut the whole thing
+            return
+        try:
+            file_path = main_file.get_path()
+            if file_path is None:
+                return
+            file_path = normalize_path(file_path, True)
+            song = self.get(file_path)
+            file_path = Path(file_path)
+            other_path = (Path(normalize_path(other_file.get_path(), True))
+                          if other_file else None)
+            if event in (Event.CREATED, Event.MOVED_IN):
+                if file_path.is_dir():
+                    self.monitor_dir(file_path)
+                    copool.add(self.scan, [str(file_path)])
+                elif not song:
+                    print_d(f"Auto-adding created file: {file_path}", self._name)
+                    self.add_filename(str(file_path))
+            elif event == Event.RENAMED:
+                if not other_path:
+                    print_w(f"No destination found for rename of {file_path}",
+                            self._name)
+                if song:
+                    print_d(f"Moving {file_path} to {other_path}...", self._name)
+                    if self.move_song(song, str(other_path)):  # type:ignore
+                        print_w(f"Song {file_path} has gone")
+                elif self.is_monitored_dir(file_path):
+                    if self.librarian:
+                        print_d(f"Moving tracks from {file_path} -> {other_path}...",
+                                self._name)
+                        copool.add(self.librarian.move_root,
+                                   str(file_path), str(other_path),
+                                   write_files=False,
+                                   priority=GLib.PRIORITY_DEFAULT)
+                    self.unmonitor_dir(file_path)
+                    if other_path:
+                        self.monitor_dir(other_path)
+                else:
+                    if other_path:
+                        print_w(f"Seems {file_path} is not a track (deleted?)",
+                                self._name)
+                        # On some (Windows?) systems CHANGED is called which can remove
+                        # before we get here, so let's try adding the new path back
+                        self.add_filename(other_path)
+
+            elif event == Event.CHANGED:
+                if song:
+                    # QL created (or knew about) this one; still check if it changed
+                    if not song.valid():
+                        self.reload(song)
+                else:
+                    print_d(f"Auto-adding new file: {file_path}", self._name)
+                    self.add_filename(file_path)
+            elif event in (Event.MOVED_OUT, Event.DELETED):
+                if song:
+                    print_d(f"...so deleting {file_path}", self._name)
+                    self.reload(song)
+                else:
+                    # either not a song, or a song that was renamed by QL
+                    if self.is_monitored_dir(file_path):
+                        self.unmonitor_dir(file_path)
+
+                    # And try to remove all songs under that dir. Slowly.
+                    gone = set()
+                    for key, song in self.iteritems():
+                        if file_path in Path(key).parents:
+                            gone.add(song)
+                    if gone:
+                        print_d(f"Removing {len(gone)} contained songs in {file_path}",
+                                self._name)
+                        actually_gone = self.remove(gone)
+                        if gone != actually_gone:
+                            print_w(f"Couldn't remove all: {gone - actually_gone}",
+                                    self._name)
+            else:
+                print_d(f"Unhandled event {event} on {file_path} ({other_path})",
+                        self._name)
+                return
+        except Exception:
+            print_w("Failed to run file monitor callback", self._name)
+            print_exc()
+        print_d(f"Finished handling {event}", self._name)
+
+    def is_monitored_dir(self, path: Path) -> bool:
+        return path in self._monitors
+
+    def unmonitor_dir(self, path: Path) -> None:
+        """Disconnect and remove any monitor for a directory, if found"""
+
+        monitor, handler_id = self._monitors.get(path, (None, None))
+        if not monitor:
+            print_d(f"Couldn't find path {path} in active monitors", self._name)
+            return
+        monitor.disconnect(handler_id)
+        del self._monitors[path]
+
+    def start_watching(self, paths: Iterable[fsnative]):
+        print_d(f"Setting up file watches on {paths}...", self._name)
+        exclude_dirs = [e for e in get_exclude_dirs() if e]
+
+        def watching_producer():
+            # TODO: integrate this better with scanning.
+            for fullpath in paths:
+                desc = _("Adding watches for %s") % (fsn2text(unexpand(fullpath)))
+                with Task(_("Library"), desc) as task:
+                    normalised = Path(normalize_path(fullpath, True)).expanduser()
+                    if any(Path(exclude) in normalised.parents
+                           for exclude in exclude_dirs):
+                        continue
+                    unpulsed = 0
+                    self.monitor_dir(normalised)
+                    for path, dirs, files in os.walk(normalised):
+                        normalised = Path(normalize_path(path, True))
+                        for d in dirs:
+                            self.monitor_dir(normalised / d)
+                        unpulsed += len(dirs)
+                        if unpulsed > 50:
+                            task.pulse()
+                            unpulsed = 0
+                        yield
+
+        copool.add(watching_producer, funcid="watch_library")
+
+    def stop_watching(self):
+        print_d(f"Removing watches on {len(self._monitors)} dirs", self._name)
+
+        for monitor, handler_id in self._monitors.values():
+            monitor.disconnect(handler_id)
+        self._monitors.clear()
+
+    def destroy(self):
+        self.stop_watching()
+        super().destroy()
