@@ -1,19 +1,17 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012,2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import sys
 import os
-
 from optparse import OptionParser
 
 import quodlibet
 from quodlibet import const
 from quodlibet.util.dprint import print_
-from quodlibet.senf import argv as sys_argv
 
 from .base import Command, CommandError
 from . import commands
@@ -41,7 +39,7 @@ def _print_help(main_cmd, parser, file=None):
 
 def main(argv=None):
     if argv is None:
-        argv = sys_argv
+        argv = sys.argv
 
     quodlibet.init_cli()
 
