@@ -7,6 +7,8 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+from typing import Optional, Type
+
 from gi.repository import Gtk, GObject
 from senf import fsn2text
 
@@ -28,12 +30,12 @@ class OverwriteWarning(WarningMessage):
     def __init__(self, parent, song):
         title = _("Tag may not be accurate")
 
-        fn_format = "<b>%s</b>" % util.escape(fsn2text(song("~basename")))
+        fn_format = util.bold(fsn2text(song("~basename")))
         description = _("%(file-name)s changed while the program was running. "
             "Saving without refreshing your library may "
             "overwrite other changes to the song.") % {"file-name": fn_format}
 
-        super(OverwriteWarning, self).__init__(
+        super().__init__(
             parent, title, description, buttons=Gtk.ButtonsType.NONE)
 
         self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
@@ -47,13 +49,11 @@ class WriteFailedError(ErrorMessage):
     def __init__(self, parent, song):
         title = _("Unable to save song")
 
-        fn_format = "<b>%s</b>" % util.escape(fsn2text(song("~basename")))
-        description = _("Saving %(file-name)s failed. The file may be "
-            "read-only, corrupted, or you do not have "
-            "permission to edit it.") % {"file-name": fn_format}
-
-        super(WriteFailedError, self).__init__(
-            parent, title, description)
+        fn_format = util.bold(fsn2text(song("~basename")))
+        description = _("Saving %(file-name)s failed."
+                        "The file may be read-only, corrupted, or you do not have "
+                        "permission to edit it.") % {"file-name": fn_format}
+        super().__init__(parent, title, description, escape_desc=False)
 
 
 class EditingPluginHandler(GObject.GObject, PluginHandler):
@@ -61,10 +61,10 @@ class EditingPluginHandler(GObject.GObject, PluginHandler):
         "changed": (GObject.SignalFlags.RUN_LAST, None, ())
     }
 
-    Kind = None
+    Kind: Optional[Type] = None
 
     def __init__(self):
-        super(EditingPluginHandler, self).__init__()
+        super().__init__()
         self.__plugins = []
 
     @property
@@ -90,10 +90,11 @@ class FilterCheckButton(ConfigCheckButton):
     __gsignals__ = {
         "preview": (GObject.SignalFlags.RUN_LAST, None, ())
         }
+    _tooltip = None
 
     def __init__(self):
-        super(FilterCheckButton, self).__init__(
-            self._label, self._section, self._key)
+        super().__init__(
+            self._label, self._section, self._key, tooltip=self._tooltip)
         try:
             self.set_active(config.getboolean(self._section, self._key))
         except:
@@ -125,7 +126,7 @@ class FilterPluginBox(Gtk.VBox):
     }
 
     def __init__(self, plugin_handler, filter_types=None):
-        super(FilterPluginBox, self).__init__()
+        super().__init__()
 
         # static filters
         if filter_types is None:
