@@ -1,14 +1,15 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig, Michael Urman
-#           2016 Nick Boultbee
+#        2016-23 Nick Boultbee
+#           2018 Peter Strulo
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk, Pango
 
-from quodlibet import config
+from quodlibet import config, print_d
 from quodlibet import util
 from quodlibet import browsers
 from quodlibet import app
@@ -21,7 +22,7 @@ from quodlibet.qltk.window import Window, PersistentWindowMixin
 from quodlibet.util.library import background_filter
 
 
-class FilterMenu(object):
+class FilterMenu:
     MENU = """
     <menu action='Filters'>
         <menuitem action='FilterGenre' always-show-image='true'/>
@@ -132,9 +133,9 @@ class FilterMenu(object):
         name = menuitem.get_name()
 
         if name == "PlayedRecently":
-            self._make_query("#(lastplayed < 7 days ago)")
+            self._make_query(u"#(lastplayed < 7 days ago)")
         elif name == "AddedRecently":
-            self._make_query("#(added < 7 days ago)")
+            self._make_query(u"#(added < 7 days ago)")
         elif name == "TopRated":
             bg = background_filter()
             songs = (bg and filter(bg, self._library)) or self._library
@@ -143,15 +144,16 @@ class FilterMenu(object):
                 return
             songs.sort()
             if len(songs) < 40:
-                self._make_query("#(playcount > %d)" % (songs[0] - 1))
+                self._make_query(u"#(playcount > %d)" % (songs[0] - 1))
             else:
-                self._make_query("#(playcount > %d)" % (songs[-40] - 1))
+                self._make_query(u"#(playcount > %d)" % (songs[-40] - 1))
         elif name == "All":
             self._browser.unfilter()
 
     def _make_query(self, query):
+        assert isinstance(query, str)
         if self._browser.can_filter_text():
-            self._browser.filter_text(query.encode('utf-8'))
+            self._browser.filter_text(query)
             self._browser.activate()
 
     def _hide_menus(self):
@@ -228,10 +230,8 @@ class LibraryBrowser(Window, util.InstanceTracker, PersistentWindowMixin):
         so we can restore them on start.
         """
 
-        names = []
-        for browser in cls.instances():
-            names.append(browser.name)
-        config.set("memory", "open_browsers", "\n".join(names))
+        config.set("memory", "open_browsers",
+                   "\n".join(browser.name for browser in cls.instances()))
 
     @classmethod
     def restore(cls, library, player):
@@ -244,7 +244,7 @@ class LibraryBrowser(Window, util.InstanceTracker, PersistentWindowMixin):
             browser.show_maybe()
 
     def __init__(self, Kind, library, player):
-        super(LibraryBrowser, self).__init__(dialog=False)
+        super().__init__(dialog=False)
         self._register_instance()
         self.name = Kind.__name__
 
@@ -256,6 +256,7 @@ class LibraryBrowser(Window, util.InstanceTracker, PersistentWindowMixin):
         view = SongList(library, update=True)
         view.info.connect("changed", self.__set_totals)
         self.songlist = view
+        self.songlist.sortable = not Kind.can_reorder
 
         sw = ScrolledWindow()
         sw.set_shadow_type(Gtk.ShadowType.IN)
@@ -314,7 +315,8 @@ class LibraryBrowser(Window, util.InstanceTracker, PersistentWindowMixin):
         if browser.background:
             bg = background_filter()
             if bg:
-                songs = filter(bg, songs)
+                songs = list(filter(bg, songs))
+        print_d(f"Setting {len(songs)} songs...")
         self.songlist.set_songs(songs, sorted)
 
     def __enqueue(self, view, path, column, player):
@@ -349,6 +351,5 @@ class LibraryBrowser(Window, util.InstanceTracker, PersistentWindowMixin):
     def __set_totals(self, info, songs):
         i = len(songs)
         length = sum(song.get("~#length", 0) for song in songs)
-        t = self.browser.status_text(count=i,
-                                     time=util.format_time_preferred(length))
+        t = self.browser.status_text(count=i, time=util.format_time_preferred(length))
         self.__statusbar.set_text(t)
