@@ -1,20 +1,20 @@
-# -*- coding: utf-8 -*-
-# Copyright 2004-2013 Joe Wreschnig, Michael Urman, Iñigo Serna,
-# Christoph Reiter, Nick Boultbee, Simonas Kazlauskas
+# Copyright 2004-2017 Joe Wreschnig, Michael Urman, Iñigo Serna,
+#                     Christoph Reiter, Nick Boultbee, Simonas Kazlauskas
+#           2018-2019 Fredrik Strupe
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk, GLib, Gdk, GdkPixbuf, Gio, GObject
+from senf import fsnative
 
 from quodlibet import qltk
-from quodlibet import config
-from quodlibet.util import thumbnails
-from quodlibet.util.path import is_fsnative
-from quodlibet.qltk.image import (get_scale_factor, pixbuf_from_file,
-    set_image_from_pbosf, get_pbosf_for_pixbuf, pbosf_render)
-from quodlibet.util.cover.manager import cover_plugins
+from quodlibet import app
+from quodlibet.util import thumbnails, print_w
+from quodlibet.qltk.image import pixbuf_from_file, \
+    calc_scale_size, scale, add_border_widget, get_surface_for_pixbuf
 
 
 # TODO: neater way of managing dependency on this particular plugin
@@ -22,48 +22,26 @@ ALBUM_ART_PLUGIN_ID = "Download Album Art"
 
 
 class BigCenteredImage(qltk.Window):
-    """Load an image and display it, scaling down to 1/2 the screen's
-    dimensions if necessary.
+    """Load an image and display it, scaling it down to the parent window size.
+    """
 
-    This might leak memory, but it could just be Python's GC being dumb."""
-
-    def __init__(self, title, fileobj, parent):
-        super(BigCenteredImage, self).__init__(type=Gtk.WindowType.POPUP)
+    def __init__(self, title, fileobj, parent, scale=0.5):
+        super().__init__(type=Gtk.WindowType.POPUP)
+        self.set_type_hint(Gdk.WindowTypeHint.TOOLTIP)
 
         assert parent
         parent = qltk.get_top_parent(parent)
         self.set_transient_for(parent)
 
-        if qltk.is_wayland():
-            # no screen size with wayland, the parent window is
-            # the next best thing..
-            width, height = parent.get_size()
-            width = int(width / 1.1)
-            height = int(height / 1.1)
-        else:
-            width = int(Gdk.Screen.width() / 1.75)
-            height = int(Gdk.Screen.height() / 1.75)
-
         self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
 
-        scale_factor = get_scale_factor(self)
-
-        pixbuf = None
-        try:
-            pixbuf = pixbuf_from_file(fileobj, (width, height), scale_factor)
-        except GLib.GError:
-            pass
-
-        # failed to load, abort
-        if not pixbuf:
+        #If image fails to set, abort construction.
+        if not self.set_image(fileobj, parent, scale):
             self.destroy()
             return
 
-        image = Gtk.Image()
-        set_image_from_pbosf(image, get_pbosf_for_pixbuf(self, pixbuf))
-
         event_box = Gtk.EventBox()
-        event_box.add(image)
+        event_box.add(self.__image)
 
         frame = Gtk.Frame()
         frame.set_shadow_type(Gtk.ShadowType.OUT)
@@ -73,14 +51,41 @@ class BigCenteredImage(qltk.Window):
 
         event_box.connect('button-press-event', self.__destroy)
         event_box.connect('key-press-event', self.__destroy)
+
         self.get_child().show_all()
 
+    def set_image(self, file, parent, scale=0.5):
+        scale_factor = self.get_scale_factor()
+
+        (width, height) = self.__calculate_screen_width(parent, scale)
+
+        pixbuf = None
+        try:
+            pixbuf = pixbuf_from_file(file, (width, height), scale_factor)
+        except GLib.GError:
+            return False
+
+        # failed to load, abort
+        if not pixbuf:
+            return False
+
+        self.__image = Gtk.Image()
+        self.__image.set_from_surface(get_surface_for_pixbuf(self, pixbuf))
+
+        return True
+
+    def __calculate_screen_width(self, parent, scale=0.5):
+        width, height = parent.get_size()
+        width = int(width * scale)
+        height = int(height * scale)
+        return (width, height)
+
     def __destroy(self, *args):
         self.destroy()
 
 
 def get_no_cover_pixbuf(width, height, scale_factor=1):
-    """A no cover pixbux at max width x height"""
+    """A no-cover pixbuf at maximum width x height"""
 
     # win32 workaround: https://bugzilla.gnome.org/show_bug.cgi?id=721062
 
@@ -116,7 +121,7 @@ class ResizeImage(Gtk.Bin):
             path = None
         else:
             path = fileobj.name
-            assert is_fsnative(path)
+            assert isinstance(path, fsnative)
 
         # XXX: Don't reload if the file path is the same.
         # Could prevent updates if fileobj.name isn't defined
@@ -133,7 +138,7 @@ class ResizeImage(Gtk.Bin):
             return self._pixbuf
         self._dirty = False
 
-        max_size = 256 * get_scale_factor(self)
+        max_size = 256 * self.get_scale_factor()
 
         self._pixbuf = None
         if self._file:
@@ -150,9 +155,7 @@ class ResizeImage(Gtk.Bin):
         if not pixbuf:
             return 0, 0
         width, height = pixbuf.get_width(), pixbuf.get_height()
-        return thumbnails.calc_scale_size(
-                (max_width, max_height),
-                (width, height))
+        return calc_scale_size((max_width, max_height), (width, height))
 
     def do_get_request_mode(self):
         if self._resize:
@@ -189,26 +192,26 @@ class ResizeImage(Gtk.Bin):
         alloc = self.get_allocation()
         width, height = alloc.width, alloc.height
 
-        scale_factor = get_scale_factor(self)
+        scale_factor = self.get_scale_factor()
 
         width *= scale_factor
         height *= scale_factor
 
         if self._path:
-            if width < 2 or height < 2:
+            if width < (2 * scale_factor) or height < (2 * scale_factor):
                 return
-            round_thumbs = config.getboolean("albumart", "round")
-            pixbuf = thumbnails.scale(
+            pixbuf = scale(
                 pixbuf, (width - 2 * scale_factor, height - 2 * scale_factor))
-            pixbuf = thumbnails.add_border_widget(
-                pixbuf, self, None, round_thumbs)
+            pixbuf = add_border_widget(pixbuf, self)
         else:
-            pixbuf = thumbnails.scale(pixbuf, (width, height))
+            pixbuf = scale(pixbuf, (width, height))
 
         style_context = self.get_style_context()
-
-        pbosf = get_pbosf_for_pixbuf(self, pixbuf)
-        pbosf_render(style_context, cairo_context, pbosf, 0, 0)
+        if not pixbuf:
+            print_w(f"Failed to scale pixbuf for {self._path}")
+            return
+        surface = get_surface_for_pixbuf(self, pixbuf)
+        Gtk.render_icon_surface(style_context, cairo_context, surface, 0, 0)
 
 
 class CoverImage(Gtk.EventBox):
@@ -221,15 +224,16 @@ class CoverImage(Gtk.EventBox):
     }
 
     def __init__(self, resize=False, size=70, song=None):
-        super(CoverImage, self).__init__()
+        super().__init__()
         self.set_visible_window(False)
         self.__song = None
         self.__file = None
         self.__current_bci = None
         self.__cancellable = None
+        self._scale = 0.9
 
         self.add(ResizeImage(resize, size))
-        self.connect('button-press-event', self.__show_cover)
+        self.connect('button-press-event', self.__album_clicked)
         self.set_song(song)
         self.get_child().show_all()
 
@@ -244,7 +248,7 @@ class CoverImage(Gtk.EventBox):
         self.set_image(None)
         if self.__cancellable:
             self.__cancellable.cancel()
-        cancellable = self.__cancellable = Gio.Cancellable.new()
+        cancellable = self.__cancellable = Gio.Cancellable()
 
         if song:
             def cb(success, result):
@@ -252,41 +256,57 @@ class CoverImage(Gtk.EventBox):
                     try:
                         self.set_image(result)
                         self.emit('cover-visible', success)
+                        self.update_bci(result)
                         # If this widget is already 'destroyed', we will get
                         # following error.
                     except AttributeError:
                         pass
-            cover_plugins.acquire_cover(cb, cancellable, song)
+                else:
+                    self.update_bci(None)
+            app.cover_manager.acquire_cover(cb, cancellable, song)
 
     def refresh(self):
         self.set_song(self.__song)
 
+    def update_bci(self, albumfile):
+        # If there's a big image displaying, it should update.
+        if self.__current_bci is not None:
+            self.__current_bci.destroy()
+            if albumfile:
+                if self._scale:
+                    self.__show_cover(self.__song, self._scale)
+                else:
+                    self.__show_cover(self.__song)
+
     def __nonzero__(self):
         return bool(self.__file)
 
     def __reset_bci(self, bci):
         self.__current_bci = None
 
-    def __show_cover(self, box, event):
-        """Show the cover as a detached BigCenteredImage.
-        If one is already showing, destroy it instead
-        If there is no image, run the AlbumArt plugin
-        """
-
+    def __album_clicked(self, box, event):
         song = self.__song
         if not song:
             return
 
-        if event.button != Gdk.BUTTON_PRIMARY or \
-                event.type != Gdk.EventType.BUTTON_PRESS:
-            return
+        if (event.type != Gdk.EventType.BUTTON_PRESS or
+                event.button == Gdk.BUTTON_MIDDLE):
+            return False
 
+        return self.__show_cover(song, scale=self._scale)
+
+    def __show_cover(self, song, scale=0.5):
+        """Show the cover as a detached BigCenteredImage.
+        If one is already showing, destroy it instead
+        If there is no image, run the AlbumArt plugin
+        """
         if not self.__file and song.is_file:
             from quodlibet.qltk.songsmenu import SongsMenu
             from quodlibet import app
 
             SongsMenu.plugins.handle(ALBUM_ART_PLUGIN_ID, app.library,
                                      qltk.get_top_parent(self), [song])
+
             return True
 
         if self.__current_bci is not None:
@@ -299,7 +319,7 @@ class CoverImage(Gtk.EventBox):
 
         try:
             self.__current_bci = BigCenteredImage(
-                song.comma("album"), self.__file, parent=self)
+                song.comma("album"), self.__file, parent=self, scale=scale)
         except GLib.GError: # reload in case the image file is gone
             self.refresh()
         else:
