@@ -1,24 +1,21 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005 Eduardo Gonzalez, Joe Wreschnig
-#           2017 Nick Boultbee
+#           2017-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
-# FIXME:
-# - Too many buttons -- saving should be automatic?
+# FIXME: Too many buttons -- saving should be automatic?
 
 import os
-import threading
+from urllib.parse import quote
 
 from gi.repository import Gtk
 
 from quodlibet import _, print_d, print_w, app
 from quodlibet import qltk
 from quodlibet import util
-from quodlibet.compat import quote
 from quodlibet.errorreport import errorhook
 from quodlibet.formats import AudioFileError
 from quodlibet.qltk import Icons
@@ -27,15 +24,14 @@ from quodlibet.util import connect_obj
 
 class LyricsPane(Gtk.VBox):
     def __init__(self, song):
-        super(LyricsPane, self).__init__(spacing=12)
+        super().__init__(spacing=12)
         self.set_border_width(12)
         view = Gtk.TextView()
         sw = Gtk.ScrolledWindow()
         sw.add(view)
         save = qltk.Button(_("_Save"), Icons.DOCUMENT_SAVE)
         delete = qltk.Button(_("_Delete"), Icons.EDIT_DELETE)
-        view_online = qltk.Button(_("_View online"),
-                                  Icons.APPLICATION_INTERNET)
+        view_online = qltk.Button(_("_View online"), Icons.APPLICATION_INTERNET)
         view.set_wrap_mode(Gtk.WrapMode.WORD)
         sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
 
@@ -66,19 +62,19 @@ class LyricsPane(Gtk.VBox):
         connect_obj(buffer, 'changed', save.set_sensitive, True)
 
     def __view_online(self, add, song):
-        artist = song.comma('artist').encode('utf-8')
-        title = song.comma('title').encode('utf-8')
+        # TODO: make this modular and plugin-friendly (#54, #3642 etc)
+        def sanitise(s: str) -> str:
+            return quote(s.replace(" ", "-")
+                         .replace(".", "")
+                         .replace("'", "")
+                         .replace('"', "")
+                         .replace(",", "-")
+                         .lower()
+                         .encode('utf-8'))
 
-        util.website("http://lyrics.wikia.com/%s:%s"
-                     % (quote(artist), quote(title)))
-
-    def __refresh(self, refresh, add, buffer, song):
-        buffer.set_text(_(u"Searching for lyrics…"))
-        refresh.set_sensitive(False)
-        thread = threading.Thread(
-            target=self.__search, args=(song, buffer, refresh, add))
-        thread.setDaemon(True)
-        thread.start()
+        artist = sanitise(song.list('artist')[0])
+        title = sanitise(song.comma('title'))
+        util.website(f"https://genius.com/{artist}-{title}-lyrics")
 
     def __save(self, save, song, buffer, delete):
         start, end = buffer.get_bounds()
@@ -89,27 +85,36 @@ class LyricsPane(Gtk.VBox):
 
     def _save_lyrics(self, song, text):
         # First, try writing to the tags.
-        song["lyrics"] = text
+        if "lyrics" not in song and "unsyncedlyrics" in song:
+            tag = "unsyncedlyrics"
+        else:
+            tag = "lyrics"
+        song[tag] = text
         try:
             song.write()
         except AudioFileError as e:
-            print_w("Couldn't write embedded lyrics (%s)" % e)
+            print_w(f"Couldn't write embedded lyrics ({e!r})")
             self._save_to_file(song, text)
         else:
-            print_d("Wrote embedded lyrics into %s" % song("~filename"))
+            print_d(f"Wrote embedded lyrics into {song('~filename')}")
             app.librarian.emit('changed', [song])
-            self._delete_file(song.lyric_filename)
+            fn = song.lyric_filename
+            if fn:
+                self._delete_file(fn)
 
     def _save_to_file(self, song, text):
-        lyricname = song.lyric_filename
+        lyric_fn = song.lyric_filename
+        if not lyric_fn:
+            print_w("No lyrics file to save to, ignoring.")
+            return
         try:
-            os.makedirs(os.path.dirname(lyricname), exist_ok=True)
+            os.makedirs(os.path.dirname(lyric_fn), exist_ok=True)
         except EnvironmentError:
             errorhook()
         try:
-            with open(lyricname, "wb") as f:
+            with open(lyric_fn, "wb") as f:
                 f.write(text.encode("utf-8"))
-            print_d("Saved lyrics to file (%s)" % lyricname)
+            print_d(f"Saved lyrics to file {lyric_fn!r}")
         except EnvironmentError:
             errorhook()
 
@@ -126,14 +131,16 @@ class LyricsPane(Gtk.VBox):
         save.set_sensitive(True)
 
     def _delete_file(self, filename):
+        if not filename:
+            return
         try:
             os.unlink(filename)
-            print_d("Removed lyrics file '%s'" % filename)
+            print_d(f"Removed lyrics file {filename!r}")
         except EnvironmentError:
             pass
         lyric_dir = os.path.dirname(filename)
         try:
             os.rmdir(lyric_dir)
-            print_d("Removed lyrics directory '%s'" % lyric_dir)
+            print_d(f"Removed lyrics directory {lyric_dir}")
         except EnvironmentError:
             pass
