@@ -1,20 +1,25 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig
 #           2012 Christoph Reiter
 #           2014 Jan Path
-#      2011-2016 Nick Boultbee
+#      2011-2023 Nick Boultbee
+#           2018 David Morris
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+from typing import List, Optional, Sequence, Tuple
 
 from gi.repository import Gtk, GLib, Gdk, GObject
+from senf import uri2fsn
 
-from quodlibet import app
+from quodlibet import app, print_w, print_d
 from quodlibet import config
 from quodlibet import const
 from quodlibet import qltk
 from quodlibet import util
+from quodlibet import _
 
 from quodlibet.query import Query
 from quodlibet.pattern import Pattern
@@ -24,22 +29,22 @@ from quodlibet.qltk.views import AllTreeView, DragScroll
 from quodlibet.qltk.ratingsmenu import ConfirmRateMultipleDialog
 from quodlibet.qltk.songmodel import PlaylistModel
 from quodlibet.qltk import Icons
-from quodlibet.util.uri import URI
+from quodlibet.qltk.util import GSignals
+from quodlibet.qltk.delete import trash_songs
 from quodlibet.formats._audio import TAG_TO_SORT, AudioFile
 from quodlibet.qltk.x import SeparatorMenuItem
-from quodlibet.qltk.songlistcolumns import create_songlist_column
+from quodlibet.qltk.songlistcolumns import create_songlist_column, SongListColumn
 from quodlibet.util import connect_destroy
 
-
 DND_QL, DND_URI_LIST = range(2)
 
 
-class SongInfoSelection(GObject.Object):
+class SongSelectionInfo(GObject.Object):
     """
     Songs which get included in the status bar summary.
 
     The `changed` signal gets fired after any of the songs in the
-    selection or the selection it self has changed.
+    selection or the selection itself has changed.
     The signal is async.
 
     Two selection states:
@@ -57,26 +62,31 @@ class SongInfoSelection(GObject.Object):
             ignore it for now.
     """
 
-    __gsignals__ = {
+    __gsignals__: GSignals = {
         # changed(songs:list)
         'changed': (GObject.SignalFlags.RUN_LAST, None, (object,))
     }
 
     def __init__(self, songlist):
-        super(SongInfoSelection, self).__init__()
+        super().__init__()
 
         self.__idle = None
         self.__songlist = songlist
         self.__selection = sel = songlist.get_selection()
         self.__count = sel.count_selected_rows()
-        self.__sel_id = songlist.connect(
-            'selection-changed', self.__selection_changed_cb)
+        self.__sel_id = songlist.connect('selection-changed', self.__selection_changed)
+        self.__sel_id = songlist.connect('songs-removed', self.__songs_removed)
 
     def destroy(self):
         self.__songlist.disconnect(self.__sel_id)
         if self.__idle:
             GLib.source_remove(self.__idle)
 
+    def refresh(self):
+        songlist = self.__songlist
+        songs = songlist.get_selected_songs() or songlist.get_songs()
+        self.emit('changed', songs)
+
     def _update_songs(self, songs):
         """After making changes (filling the list) call this to
         skip any queued changes and emit the passed songs instead"""
@@ -98,7 +108,13 @@ class SongInfoSelection(GObject.Object):
         self.__idle = GLib.idle_add(
             self.__idle_emit, songs, priority=GLib.PRIORITY_LOW)
 
-    def __selection_changed_cb(self, songlist, selection):
+    def __songs_removed(self, songlist, removed):
+        try:
+            self.__emit_info_selection()
+        except Exception as e:
+            print_w(f"Couldn't process removed songs ({e})r")
+
+    def __selection_changed(self, songlist, selection):
         count = selection.count_selected_rows()
         if self.__count == count == 0:
             return
@@ -113,20 +129,8 @@ class SongInfoSelection(GObject.Object):
 def get_columns():
     """Gets the list of songlist column headings"""
 
-    if config.has_option("settings", "columns"):
-        columns = config.getstringlist(
-            "settings", "columns", const.DEFAULT_COLUMNS)
-    else:
-        # migrate old settings
-        try:
-            columns = config.get("settings", "headers").split()
-        except config.Error:
-            columns = const.DEFAULT_COLUMNS
-        else:
-            config.remove_option("settings", "headers")
-            set_columns(columns)
-            config.setstringlist("settings", "columns", columns)
-
+    columns = config.getstringlist("settings", "columns",
+                                   const.DEFAULT_COLUMNS)
     if "~current" in columns:
         columns.remove("~current")
     return columns
@@ -147,6 +151,8 @@ def get_sort_tag(tag):
     replace_order = {
         "~#track": "",
         "~#disc": "",
+        "~#tracks": "",
+        "~#discs": "",
         "~length": "~#length"
     }
 
@@ -156,11 +162,13 @@ def get_sort_tag(tag):
         tag = "album"
 
     if "<" in tag:
-        for key, value in replace_order.iteritems():
-            tag = tag.replace("<%s>" % key, "<%s>" % value)
-        for key, value in TAG_TO_SORT.iteritems():
+        for key, value in replace_order.items():
+            if value:
+                value = f"<{value}>"
+            tag = tag.replace(f"<{key}>", value)
+        for key, value in TAG_TO_SORT.items():
             tag = tag.replace("<%s>" % key,
-                               "<{1}|<{1}>|<{0}>>".format(key, value))
+                              "<{1}|<{1}>|<{0}>>".format(key, value))
         tag = Pattern(tag).format
     else:
         tags = util.tagsplit(tag)
@@ -188,15 +196,23 @@ def header_tag_split(header):
         return util.tagsplit(header)
 
 
-class SongListDnDMixin(object):
+class SongListDnDMixin(GObject.GObject):
     """DnD support for the SongList class"""
 
     def setup_drop(self, library):
+        self.connect('drag-begin', self.__drag_begin)
         self.connect('drag-motion', self.__drag_motion)
         self.connect('drag-leave', self.__drag_leave)
         self.connect('drag-data-get', self.__drag_data_get)
         self.connect('drag-data-received', self.__drag_data_received, library)
 
+    def __drag_begin(self, *args):
+        ok, state = Gtk.get_current_event_state()
+        if ok and state & qltk.get_primary_accel_mod():
+            self.__force_copy = True
+        else:
+            self.__force_copy = False
+
     def enable_drop(self, by_row=True):
         targets = [
             ("text/x-quodlibet-songs", Gtk.TargetFlags.SAME_APP, DND_QL),
@@ -209,6 +225,7 @@ class SongListDnDMixin(object):
         self.drag_dest_set(Gtk.DestDefaults.ALL, targets,
                            Gdk.DragAction.COPY | Gdk.DragAction.MOVE)
         self.__drop_by_row = by_row
+        self.__force_copy = False
 
     def disable_drop(self):
         targets = [
@@ -224,7 +241,8 @@ class SongListDnDMixin(object):
         if self.__drop_by_row:
             self.set_drag_dest(x, y)
             self.scroll_motion(x, y)
-            if Gtk.drag_get_source_widget(ctx) == self:
+            if Gtk.drag_get_source_widget(ctx) == self and \
+                not self.__force_copy:
                 kind = Gdk.DragAction.MOVE
             else:
                 kind = Gdk.DragAction.COPY
@@ -253,8 +271,18 @@ class SongListDnDMixin(object):
                 return
 
             qltk.selection_set_songs(sel, songs)
-            if ctx.get_actions() & Gdk.DragAction.MOVE:
-                self.__drag_iters = map(model.get_iter, paths)
+
+            # DEM 2018/05/25: The below check is a deliberate repetition of
+            # code in the drag-motion signal handler.  In MacOS/Quartz, the
+            # context action is not propogated between event handlers for
+            # drag-motion and drag-data-get using "ctx.get_actions()".  It is
+            # unclear if this is a bug or expected behavior.  Regardless, the
+            # context widget information is the same so identical behavior can
+            # be achieved by simply using the same widget check as in the move
+            # action.
+            if Gtk.drag_get_source_widget(ctx) == self and \
+                not self.__force_copy:
+                self.__drag_iters = list(map(model.get_iter, paths))
             else:
                 self.__drag_iters = []
         else:
@@ -266,28 +294,30 @@ class SongListDnDMixin(object):
         model = view.get_model()
         if info == DND_QL:
             filenames = qltk.selection_get_filenames(sel)
-            move = (Gtk.drag_get_source_widget(ctx) == view)
+            move = bool(ctx.get_selected_action() & Gdk.DragAction.MOVE)
         elif info == DND_URI_LIST:
             def to_filename(s):
                 try:
-                    return URI(s).filename
+                    return uri2fsn(s)
                 except ValueError:
                     return None
 
-            filenames = filter(None, map(to_filename, sel.get_uris()))
+            filenames = list(filter(None, map(
+                to_filename, sel.get_uris())))
             move = False
         else:
             Gtk.drag_finish(ctx, False, False, etime)
             return
-
+        # Should always have one here, but you never know (also: types)
+        librarian = library.librarian or library
         to_add = []
         for filename in filenames:
-            if filename not in library.librarian:
+            if filename not in librarian:
                 library.add_filename(filename)
             elif filename not in library:
-                to_add.append(library.librarian[filename])
+                to_add.append(librarian[filename])
         library.add(to_add)
-        songs = filter(None, map(library.get, filenames))
+        songs: List = list(filter(None, map(library.get, filenames)))
         if not songs:
             Gtk.drag_finish(ctx, bool(not filenames), False, etime)
             return
@@ -304,7 +334,7 @@ class SongListDnDMixin(object):
             position = Gtk.TreeViewDropPosition.AFTER
 
         if move and Gtk.drag_get_source_widget(ctx) == view:
-            iter = model.get_iter(path) # model can't be empty, we're moving
+            iter = model.get_iter(path)  # model can't be empty, we're moving
             if position in (Gtk.TreeViewDropPosition.BEFORE,
                             Gtk.TreeViewDropPosition.INTO_OR_BEFORE):
                 while self.__drag_iters:
@@ -318,7 +348,7 @@ class SongListDnDMixin(object):
             try:
                 iter = model.get_iter(path)
             except ValueError:
-                iter = model.append(row=[song]) # empty model
+                iter = model.append(row=[song])  # empty model
             else:
                 if position in (Gtk.TreeViewDropPosition.BEFORE,
                                 Gtk.TreeViewDropPosition.INTO_OR_BEFORE):
@@ -334,18 +364,18 @@ class SongListDnDMixin(object):
         return window.browser.dropped(songs)
 
 
-class SongList(AllTreeView, SongListDnDMixin, DragScroll,
-               util.InstanceTracker):
-    # A TreeView containing a list of songs.
+class SongList(AllTreeView, SongListDnDMixin, DragScroll, util.InstanceTracker):
+    """A TreeView containing a list of songs."""
 
-    __gsignals__ = {
-        # changed(songs:list)
+    __gsignals__: GSignals = {
+        'songs-removed': (GObject.SignalFlags.RUN_LAST, None, (object,)),
         'orders-changed': (GObject.SignalFlags.RUN_LAST, None, [])
     }
 
-    headers = [] # The list of current headers.
+    headers: List[str] = []
+    """The list of current headers."""
+
     star = list(Query.STAR)
-    sortable = True
 
     def Menu(self, header, browser, library):
         songs = self.get_selected_songs()
@@ -356,14 +386,14 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             # Translators: The substituted string is the name of the
             # selected column (a translated tag name).
             b = qltk.MenuItem(
-                _("_Filter on %s") % util.tag(t, True), Icons.EDIT_SELECT_ALL)
+                _("_Filter on %s") % util.tag(t, True), Icons.EDIT_FIND)
             b.connect('activate', self.__filter_on, t, songs, browser)
             return b
 
         header = header_tag_split(header)[0]
         can_filter = browser.can_filter
         menu_items = []
-        if (header not in ["artist", "album"] and can_filter(header)):
+        if header not in ["artist", "album"] and can_filter(header):
             menu_items.append(Filter(header))
         if can_filter("artist"):
             menu_items.append(Filter("artist"))
@@ -374,12 +404,13 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         menu.show_all()
         return menu
 
-    def __init__(self, library, player=None, update=False,
-                 model_cls=PlaylistModel):
-        super(SongList, self).__init__()
+    def __init__(self, library, player=None, update=False, model_cls=PlaylistModel,
+                 sortable: bool = True):
+        super().__init__()
+        self._sortable = sortable
         self._register_instance(SongList)
         self.set_model(model_cls())
-        self.info = SongInfoSelection(self)
+        self.info = SongSelectionInfo(self)
         self.set_size_request(200, 150)
         self.set_rules_hint(True)
         self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
@@ -388,7 +419,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         self._first_column = None
         # A priority list of how to apply the sort keys.
         # might contain column header names not present...
-        self._sort_sequence = []
+        self._sort_sequence: List[str] = []
         self.set_column_headers(self.headers)
         librarian = library.librarian or library
 
@@ -399,15 +430,12 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             connect_destroy(librarian, 'added', self.__song_added)
 
         if player:
-            connect_destroy(
-                player, 'paused', lambda *x: self.__redraw_current())
-            connect_destroy(
-                player, 'unpaused', lambda *x: self.__redraw_current())
-            connect_destroy(
-                player, 'error', lambda *x: self.__redraw_current())
+            connect_destroy(player, 'paused', lambda *x: self.__redraw_current())
+            connect_destroy(player, 'unpaused', lambda *x: self.__redraw_current())
+            connect_destroy(player, 'error', lambda *x: self.__redraw_current())
 
-        self.connect('button-press-event', self.__button_press, librarian)
-        self.connect('key-press-event', self.__key_press, librarian)
+        self.connect('button-press-event', self.__button_press, library)
+        self.connect('key-press-event', self.__key_press, library, player)
 
         self.setup_drop(library)
         self.disable_drop()
@@ -417,7 +445,19 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         self.connect('destroy', self.__destroy)
 
     @property
-    def model(self):
+    def sortable(self) -> bool:
+        """Whether the columns clicked to enable sorting of songs"""
+        return self._sortable
+
+    @sortable.setter
+    def sortable(self, value: bool):
+        # It's either sortable or clickable columns, both ends in a buggy UI (see #4099)
+        always_sortable = config.getboolean("song_list", "always_allow_sorting")
+        self._sortable = value or always_sortable
+        self.set_headers_clickable(value)
+
+    @property
+    def model(self) -> Gtk.TreeModel:
         return self.get_model()
 
     @property
@@ -459,9 +499,8 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
                     dont_reverse = True
                     break
 
-        # set the inidicators
+        # set the indicators
         default_order = Gtk.SortType.ASCENDING
-        reversed_ = False
         for c in self.get_columns():
             if c is column:
                 if c.get_sort_indicator():
@@ -469,7 +508,6 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
                         order = c.get_sort_order()
                     else:
                         order = not c.get_sort_order()
-                        reversed_ = True
                 else:
                     order = default_order
                 c.set_sort_order(order)
@@ -481,10 +519,13 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
 
         if refresh:
             songs = self.get_songs()
-            if reversed_:
-                # python sort is faster if presorted
-                songs.reverse()
-            self.set_songs(songs, scroll_select=True)
+            song_order = self._get_song_order(songs)
+            self.model.reorder(song_order)
+
+            selection = self.get_selection()
+            _, paths = selection.get_selected_rows()
+            if paths:
+                self.scroll_to_cell(paths[0], use_align=True, row_align=0.5)
 
         self.emit("orders-changed")
 
@@ -561,7 +602,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
     def __search_func(self, model, column, key, iter, *args):
         for column in self.get_columns():
             value = model.get_value(iter)(column.header_name)
-            if not isinstance(value, basestring):
+            if not isinstance(value, str):
                 continue
             elif key in value.lower() or key in value:
                 return False
@@ -606,15 +647,14 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             precision = config.RATINGS.precision
             count = int(float(cellx - 5) / width) + 1
             rating = max(0.0, min(1.0, count * precision))
-            if (rating <= precision and
-                    song("~#rating") == precision):
+            if rating <= precision and song("~#rating") == precision:
                 rating = 0.0
             self.__set_rating(rating, [song], librarian)
 
     def __set_rating(self, value, songs, librarian):
         count = len(songs)
         if (count > 1 and
-                config.getboolean("browsers", "rating_confirm_multiple")):
+            config.getboolean("browsers", "rating_confirm_multiple")):
             dialog = ConfirmRateMultipleDialog(self, count, value)
             if dialog.run() != Gtk.ResponseType.YES:
                 return
@@ -622,22 +662,18 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             song["~#rating"] = value
         librarian.changed(songs)
 
-    def __key_press(self, songlist, event, librarian):
-        rating_accels = [
-            "<Primary>%d" % i for i in range(
-                min(10, config.RATINGS.number + 1))]
-
-        if (qltk.is_accel(event, *rating_accels) and
-                config.getboolean("browsers", "rating_hotkeys")):
-            rating = int(chr(event.keyval)) * config.RATINGS.precision
-            self.__set_rating(rating, self.get_selected_songs(), librarian)
-            return True
-        elif qltk.is_accel(event, "<Primary>Return", "<Primary>KP_Enter"):
+    def __key_press(self, songlist, event, librarian, player):
+        if qltk.is_accel(event, "<Primary>Return", "<Primary>KP_Enter"):
             self.__enqueue(self.get_selected_songs())
             return True
         elif qltk.is_accel(event, "<Primary>F"):
             self.emit('start-interactive-search')
             return True
+        elif qltk.is_accel(event, "<Primary>Delete"):
+            songs = self.get_selected_songs()
+            if songs:
+                trash_songs(self, songs, librarian)
+            return True
         elif qltk.is_accel(event, "<alt>Return"):
             songs = self.get_selected_songs()
             if songs:
@@ -650,10 +686,29 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
                 window = Information(librarian, songs, self)
                 window.show()
             return True
+        elif qltk.is_accel(event, "space", "KP_Space") and player is not None:
+            player.paused = not player.paused
+            return True
+        elif qltk.is_accel(event, "F2"):
+            songs = self.get_selected_songs()
+            if len(songs) > 1:
+                print_d("Can't edit more than one")
+            elif songs:
+                path, col = songlist.get_cursor()
+                song = self.get_first_selected_song()
+                cls = type(col).__name__
+                if col.can_edit:
+                    print_d(f"Let's edit this: {song} ({cls} can be edited)")
+                    renderers = col.get_cells()
+                    renderers[0].props.editable = True
+                    self.set_cursor(path, col, start_editing=True)
+                else:
+                    print_d(f"Can't edit {cls}. Maybe it's synthetic / numeric?")
+
         return False
 
     def __enqueue(self, songs):
-        songs = filter(lambda s: s.can_add, songs)
+        songs = [s for s in songs if s.can_add]
         if songs:
             from quodlibet import app
             app.window.playlist.enqueue(songs)
@@ -666,12 +721,12 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             model.row_changed(path, iter_)
 
     def __columns_changed(self, *args):
-        headers = map(lambda h: h.header_name, self.get_columns())
+        headers = [h.header_name for h in self.get_columns()]
         SongList.set_all_column_headers(headers)
         SongList.headers = headers
 
     def __column_width_changed(self, *args):
-        # make sure non resizable columns stay non expanding.
+        # make sure non-resizable columns stay non-expanding.
         # gtk likes to change them sometimes
         for c in self.get_columns():
             if not c.get_resizable() and c.get_expand():
@@ -711,7 +766,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         SongList.star = star
 
     def set_model(self, model):
-        super(SongList, self).set_model(model)
+        super().set_model(model)
         self.set_search_column(0)
 
     def clear(self):
@@ -729,19 +784,31 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             return []
         return model.get()
 
-    def _sort_songs(self, songs):
-        """Sort passed songs in place based on the column sort orders"""
+    def _get_song_order(self, songs: List[AudioFile]) -> Optional[Sequence[int]]:
+        """Returns mapping from new position to position in given list of songs
+        when sorted based on the column sort orders"""
 
+        orders = self.get_sort_orders()
+        if orders:
+            song_order = list(range(len(songs)))
+            for key, reverse in self.__get_song_sort_key_func(orders):
+                song_order.sort(key=lambda i: key(songs[i]), reverse=reverse)
+            return song_order
+        else:
+            return None
+
+    def __get_song_sort_key_func(self, order):
         last_tag = None
         last_order = None
         first = True
-        for tag, reverse in self.get_sort_orders():
+        key_func = []
+        for tag, reverse in order:
             tag = get_sort_tag(tag)
 
             # always sort using the default sort key first
             if first:
                 first = False
-                songs.sort(key=lambda s: s.sort_key, reverse=reverse)
+                key_func.append((lambda s: s.sort_key, reverse))
                 last_order = reverse
                 last_tag = ""
 
@@ -752,10 +819,11 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             last_tag = tag
 
             if tag == "":
-                songs.sort(key=lambda s: s.sort_key, reverse=reverse)
+                key_func.append((lambda s: s.sort_key, reverse))
             else:
                 sort_func = AudioFile.sort_by_func(tag)
-                songs.sort(key=sort_func, reverse=reverse)
+                key_func.append((sort_func, reverse))
+        return key_func
 
     def add_songs(self, songs):
         """Add songs to the list in the right order and position"""
@@ -772,16 +840,12 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             model.append_many(songs)
             return
 
-        # FIXME: Replace with something fast
-        old_songs = self.get_songs()
-        old_songs.extend(songs)
+        for song in songs:
+            insert_iter = self.__find_song_position(song)
+            model.insert_before(insert_iter, row=[song])
 
-        self._sort_songs(old_songs)
-
-        for index, song in sorted(zip(map(old_songs.index, songs), songs)):
-            model.insert(index, row=[song])
-
-    def set_songs(self, songs, sorted=False, scroll=True, scroll_select=False):
+    def set_songs(self, songs: List[AudioFile], sorted: bool = False,
+                  scroll: bool = True, scroll_select: bool = False):
         """Fill the song list.
 
         If sorted is True, the passed songs will not be sorted and
@@ -796,13 +860,15 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         model = self.get_model()
         assert model is not None
 
+        song_order = None
+
         if not sorted:
             # make sure some sorting is set and visible
             if not self.is_sorted():
                 default = self.find_default_sort_column()
                 if default:
                     self.toggle_column_sort(default, refresh=False)
-            self._sort_songs(songs)
+            song_order = self._get_song_order(songs)
         else:
             self.clear_sort()
 
@@ -812,6 +878,8 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
 
         with self.without_model() as model:
             model.set(songs)
+            if song_order:
+                model.reorder(song_order)
 
         # scroll to the first selected or current song and restore
         # selection for the first selected item if there was one
@@ -876,21 +944,86 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
 
         def func(model, path, iter_, user_data):
             songs.append(model.get_value(iter_))
+
         selection = self.get_selection()
         selection.selected_foreach(func, None)
         return songs
 
+    def __find_song_position(self, song):
+        """Finds the appropriate position of a song in a sorted song list.
+
+        Returns iter of the song after the given song according to the current
+        sort order.
+
+        Returns None if the correct position is at the end of the song list.
+        """
+
+        model = self.get_model()
+        order = self.get_sort_orders()
+        sort_key_func = list(enumerate(reversed(
+            self.__get_song_sort_key_func(order))))
+        song_sort_keys = [key(song) for i, (key, r) in sort_key_func]
+        i = 0
+        j = len(model)
+        while i < j:
+            mid = (i + j) // 2
+            other_song_iter = model.iter_nth_child(None, mid)
+            other_song = model.get_value(other_song_iter)
+            song_is_lower = False
+            for i, (key, reverse) in sort_key_func:
+                other_key = key(other_song)
+                is_lower = song_sort_keys[i] < other_key
+                is_greater = song_sort_keys[i] > other_key
+                if not reverse and is_lower or reverse and is_greater:
+                    song_is_lower = True
+                    break
+                if not reverse and is_greater or reverse and is_lower:
+                    break
+            if song_is_lower:
+                j = mid
+            else:
+                i = mid + 1
+        if i < len(model):
+            return model.iter_nth_child(None, i)
+        return None
+
+    def __find_iters_in_selection(self, songs) -> Tuple[List, List, bool]:
+        model, rows = self.get_selection().get_selected_rows()
+        rows = rows or []
+        iters = []
+        removed_songs = []
+        for r in rows:
+            if model[r][0] in songs:
+                iters.append(model[r].iter)
+                removed_songs.append(model[r][0])
+        complete = len(iters) == len(songs)
+        return iters, removed_songs, complete
+
     def __song_updated(self, librarian, songs):
         """Only update rows that are currently displayed.
         Warning: This makes the row-changed signal useless.
         """
+        model = self.get_model()
+        if config.getboolean("song_list", "auto_sort") and self.is_sorted():
+            iters, _, complete = self.__find_iters_in_selection(songs)
+
+            if not complete:
+                iters = model.find_all(songs)
+
+            rows = [Gtk.TreeRowReference.new(model, model.get_path(i))
+                    for i in iters]
+
+            for row in rows:
+                iter = model.get_iter(row.get_path())
+                song = model.get_value(iter)
+                insert_iter = self.__find_song_position(song)
+                model.move_before(iter, insert_iter)
 
         vrange = self.get_visible_range()
         if vrange is None:
             return
         (start,), (end,) = vrange
-        model = self.get_model()
-        for path in xrange(start, end + 1):
+        for path in range(start, end + 1):
             row = model[path]
             if row[0] in songs:
                 model.row_changed(row.path, row.iter)
@@ -899,61 +1032,43 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         window = qltk.get_top_parent(self)
         filter_ = window.browser.active_filter
         if callable(filter_):
-            self.add_songs(filter(filter_, songs))
+            self.add_songs(list(filter(filter_, songs)))
 
     def __song_removed(self, librarian, songs, player):
-        # The player needs to be called first so it can ge the next song
-        # in case the current one gets deleted and the order gets reset.
+        try:
+            # The player needs to be called first so it can ge the next song
+            # in case the current one gets deleted and the order gets reset.
+            if player:
+                for song in songs:
+                    player.remove(song)
 
-        if player:
-            for song in songs:
-                player.remove(song)
+            model = self.get_model()
 
-        # The selected songs are removed from the library and should
-        # be removed from the view.
+            # The selected songs are removed from the library and should
+            # be removed from the view.
 
-        if not len(self.get_model()):
-            return
-
-        songs = set(songs)
-
-        # search in the selection first
-        # speeds up common case: select songs and remove them
-        model, rows = self.get_selection().get_selected_rows()
-        rows = rows or []
-        iters = [model[r].iter for r in rows if model[r][0] in songs]
-
-        # if not all songs were in the selection, search the whole view
-        if len(iters) != len(songs):
-            iters = model.find_all(songs)
-
-        self.remove_iters(iters)
-
-    def __song_properties(self, librarian):
-        model, rows = self.get_selection().get_selected_rows()
-        if rows:
-            songs = [model[row][0] for row in rows]
-        else:
-            from quodlibet import app
-            if app.player.song:
-                songs = [app.player.song]
-            else:
+            if not len(model):
                 return
-        window = SongProperties(librarian, songs, parent=self)
-        window.show()
 
-    def __information(self, librarian):
-        model, rows = self.get_selection().get_selected_rows()
-        if rows:
-            songs = [model[row][0] for row in rows]
-        else:
-            from quodlibet import app
-            if app.player.song:
-                songs = [app.player.song]
-            else:
-                return
-        window = Information(librarian, songs, self)
-        window.show()
+            songs = set(songs)
+
+            # search in the selection first
+            # speeds up common case: select songs and remove them
+            iters, removed_songs, complete = self.__find_iters_in_selection(songs)
+
+            # if not all songs were in the selection, search the whole view
+            if not complete:
+                removed_songs = []
+                for iter_, value in self.model.iterrows():
+                    if value in songs:
+                        iters.append(iter_)
+                        removed_songs.append(value)
+
+            if removed_songs:
+                self.emit('songs-removed', set(removed_songs))
+            self.remove_iters(iters)
+        except Exception as e:
+            print_w(f"Couldn't process removed songs: {e}", self)
 
     def set_first_column_type(self, column_type):
         """Set a column that will be included at the beginning"""
@@ -990,7 +1105,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             column_expands[ce[i]] = int(ce[i + 1])
 
         for t in headers:
-            column = create_songlist_column(t)
+            column = create_songlist_column(self.model, t)
             if column.get_resizable():
                 if t in column_widths:
                     column.set_fixed_width(column_widths[t])
@@ -1018,12 +1133,12 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             column.set_reorderable(True)
             self.append_column(column)
 
-        self.columns_autosize()
         self.set_sort_orders(old_sort)
+        self.columns_autosize()
 
         self.handler_unblock(self.__csig)
 
-    def __getmenu(self, column):
+    def _menu(self, column: SongListColumn) -> Gtk.Menu:
         menu = Gtk.Menu()
 
         def selection_done_cb(menu):
@@ -1031,16 +1146,17 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
 
         menu.connect('selection-done', selection_done_cb)
 
-        current = SongList.headers[:]
-        current_set = set(current)
+        current_set = set(SongList.headers)
 
-        def tag_title(tag):
+        def tag_title(tag: str):
             if "<" in tag:
                 return util.pattern(tag)
             return util.tag(tag)
-        current = zip(map(tag_title, current), current)
 
-        def add_header_toggle(menu, pair, active, column=column):
+        current = [(tag_title(c), c) for c in SongList.headers]
+
+        def add_header_toggle(menu: Gtk.Menu, pair: Tuple[str, str], active: bool,
+                              column: SongListColumn = column):
             header, tag = pair
             item = Gtk.CheckMenuItem(label=header)
             item.tag = tag
@@ -1057,19 +1173,21 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         sep.show()
         menu.append(sep)
 
-        trackinfo = """title genre ~title~version ~#track
-            ~#playcount ~#skipcount ~rating ~#length ~playlists""".split()
+        trackinfo = """title genre comment ~title~version ~#track
+            ~#playcount ~#skipcount ~rating ~#length ~playlists
+            bpm initialkey""".split()
         peopleinfo = """artist ~people performer arranger author composer
             conductor lyricist originalartist""".split()
         albuminfo = """album ~album~discsubtitle labelid ~#disc ~#discs
             ~#tracks albumartist""".split()
         dateinfo = """date originaldate recordingdate ~year ~originalyear
             ~#laststarted ~#lastplayed ~#added ~#mtime""".split()
-        fileinfo = """~format ~#bitrate ~#filesize ~filename ~basename ~dirname
-            ~uri ~codec ~encoding""".split()
+        fileinfo = """~format ~#bitdepth ~#bitrate ~#filesize
+            ~filename ~basename ~dirname ~uri ~codec ~encoding ~#channels
+            ~#samplerate""".split()
         copyinfo = """copyright organization location isrc
             contact website""".split()
-        all_headers = sum(
+        all_headers: List[str] = sum(
             [trackinfo, peopleinfo, albuminfo, dateinfo, fileinfo, copyinfo],
             [])
 
@@ -1158,7 +1276,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         if event is not None and event.button != Gdk.BUTTON_SECONDARY:
             return False
 
-        menu = self.__getmenu(column)
+        menu = self._menu(column)
         menu.attach_to_widget(self, None)
 
         if event:
