@@ -1,9 +1,10 @@
-# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig
+#           2015 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk
 
@@ -11,7 +12,7 @@ from quodlibet.util import tag
 from quodlibet.util.tags import USER_TAGS
 
 
-class _TagsCombo(object):
+class _TagsCombo:
     __tags = sorted(USER_TAGS)
 
     def _fill_model(self, can_change):
@@ -44,7 +45,7 @@ class TagsComboBox(Gtk.ComboBox, _TagsCombo):
     The 'tag' attribute is the currently chosen tag."""
 
     def __init__(self, can_change=None):
-        super(TagsComboBox, self).__init__(model=Gtk.ListStore(str, str))
+        super().__init__(model=Gtk.ListStore(str, str))
         self._fill_model(can_change)
         self.set_active(0)
 
@@ -60,15 +61,17 @@ class TagsComboBoxEntry(Gtk.ComboBox, _TagsCombo):
 
     The 'tag' attribute is the currently chosen tag."""
 
-    def __init__(self, can_change=None):
-        super(TagsComboBoxEntry, self).__init__(
+    def __init__(self, can_change=None, tooltip_markup=None):
+        super().__init__(
             model=Gtk.ListStore(str, str),
             entry_text_column=0,
             has_entry=True)
         self._fill_model(can_change)
+        if tooltip_markup:
+            self.get_child().set_tooltip_markup(tooltip_markup)
 
     def _fill_model(self, can_change):
-        super(TagsComboBoxEntry, self)._fill_model(can_change)
+        super()._fill_model(can_change)
         comp = Gtk.EntryCompletion()
         comp.set_model(self.get_model())
         comp.set_text_column(0)
