@@ -1,6 +1,6 @@
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
-#           2013 Christoph Reiter
-#           2020 Nick Boultbee
+#                2013 Christoph Reiter
+#           2020-2021 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -9,10 +9,13 @@
 
 import os
 import time
+from typing import Collection
 
 from gi.repository import GObject, GLib
 
-from quodlibet import config
+from quodlibet import config, print_d
+from quodlibet.formats import AudioFile
+from quodlibet.library.base import Library
 
 
 class TimeTracker(GObject.GObject):
@@ -150,7 +153,11 @@ class SongTracker:
             else:
                 config.set("memory", "seek", 0)
 
-            if self.elapsed > 0.5 * song.get("~#length", 1):
+            playcount_minimum_length = config.getfloat(
+                "player", "playcount_minimum_length_proportion") * int(song.get(
+                "~#length", 1))
+
+            if self.elapsed >= playcount_minimum_length:
                 song["~#lastplayed"] = int(time.time())
                 song["~#playcount"] = song.get("~#playcount", 0) + 1
                 self.__changed(librarian, song)
@@ -168,18 +175,22 @@ class SongTracker:
 class FSInterface:
     """Provides a file in ~/.quodlibet to indicate what song is playing."""
 
-    def __init__(self, path, player):
+    def __init__(self, path, player, library: Library):
         self.path = path
         self._player = player
-        self._ids = [
+        self._pids = [
             player.connect('song-started', self.__started),
             player.connect('song-ended', self.__ended),
         ]
+        self._lids = [library.connect('changed', self.__changed)]
+        self._library = library
 
     def destroy(self):
-        for id_ in self._ids:
+        for id_ in self._pids:
             self._player.disconnect(id_)
-
+        for id_ in self._lids:
+            print_d(f"Disconnecting signal {id_} from {self._library}")
+            self._library.disconnect(id_)
         try:
             os.unlink(self.path)
         except EnvironmentError:
@@ -199,3 +210,9 @@ class FSInterface:
             os.unlink(self.path)
         except EnvironmentError:
             pass
+
+    def __changed(self, _lib: Library, songs: Collection[AudioFile]):
+        current = self._player.song
+        if current and current in songs:
+            print_d("Current song changed, updating current file")
+            self.__started(self._player, current)
