@@ -1,5 +1,5 @@
 # Copyright 2004-2009 Joe Wreschnig, Michael Urman, Steven Robertson
-#           2011-2020 Nick Boultbee
+#           2011-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@ import sys
 import unicodedata
 import threading
 import locale
-from typing import Dict, List
+from typing import Dict, List, Callable
 from functools import reduce
 
 # Windows doesn't have fcntl, just don't lock for now
@@ -25,7 +25,7 @@ try:
 except ImportError:
     fcntl = None  # type: ignore
 
-from senf import fsnative, argv
+from senf import fsnative
 
 from quodlibet.util.string.titlecase import title
 
@@ -36,7 +36,7 @@ from .misc import cached_func, get_module_dir, get_ca_file, \
 from .environment import is_plasma, is_unity, is_enlightenment, \
     is_linux, is_openbsd, is_windows, is_wine, is_osx, is_flatpak, matches_flatpak_runtime
 from .enum import enum
-from .i18n import _, C_
+from .i18n import _, ngettext, C_
 
 
 # flake8
@@ -118,7 +118,7 @@ class OptionParser:
             l = max(l, len(k) + len(self.__args.get(k, "")) + 4)
 
         s = _("Usage: %(program)s %(usage)s") % {
-            "program": argv[0],
+            "program": sys.argv[0],
             "usage": self.__usage if self.__usage else _("[options]"),
         }
         s += "\n"
@@ -155,7 +155,7 @@ class OptionParser:
 
     def parse(self, args=None):
         if args is None:
-            args = argv[1:]
+            args = sys.argv[1:]
         from getopt import getopt, GetoptError
         try:
             opts, args = getopt(args, self.__shorts(), self.__longs())
@@ -172,7 +172,7 @@ class OptionParser:
                 text.append(
                     _("%r is not a unique prefix.") % s.split()[1])
             if "help" in self.__args:
-                text.append(_("Try %s --help.") % argv[0])
+                text.append(_("Try %s --help.") % sys.argv[0])
 
             print_e("\n".join(text))
             raise SystemExit(True)
@@ -200,26 +200,30 @@ class OptionParser:
             return transopts, args
 
 
-def escape(str):
+def escape(string: str) -> str:
     """Escape a string in a manner suitable for XML/Pango."""
-    return str.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")
+    return string.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")
 
 
-def unescape(str):
+def unescape(string: str) -> str:
     """Unescape a string in a manner suitable for XML/Pango."""
-    return str.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")
+    return string.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")
 
 
-def bold(string):
-    return "<b>%s</b>" % string
+def bold(string: str, escaper: Callable[[str], str] = escape) -> str:
+    return "<b>%s</b>" % escaper(string)
 
 
-def monospace(string):
-    return "<tt>%s</tt>" % string
+def monospace(string: str, escaper: Callable[[str], str] = escape) -> str:
+    return "<tt>%s</tt>" % escaper(string)
 
 
-def italic(string):
-    return "<i>%s</i>" % string
+def italic(string: str, escaper: Callable[[str], str] = escape) -> str:
+    return "<i>%s</i>" % escaper(string)
+
+
+def bold_italic(string: str, escaper: Callable[[str], str] = escape) -> str:
+    return "<b><i>%s</i></b>" % escaper(string)
 
 
 def parse_time(timestr, err=object()):
@@ -504,7 +508,7 @@ def tag(name, cap=True):
     # Strips ~ and ~# from the start and runs it through a map (which
     # the user can configure).
     if not name:
-        return _("Invalid tag")
+        return ngettext("Invalid tag", "Invalid tags", 1)
     else:
         from quodlibet.util.tags import readable
         parts = map(readable, tagsplit(name))
@@ -957,7 +961,7 @@ def load_library(names, shared=True):
 
         try:
             return load_func(dlopen_name), name
-        except OSError as e:
+        except (OSError, AttributeError) as e:
             errors.append(str(e))
 
     raise OSError("\n".join(errors))
@@ -1091,7 +1095,7 @@ class MainRunner:
 
 
 def re_escape(string, BAD="/.^$*+-?{,\\[]|()<>#=!:"):
-    """A re.escape which also works with unicode"""
+    """A `re.escape` which also works with unicode"""
 
     needs_escape = lambda c: (c in BAD and "\\" + c) or c
     return type(string)().join(map(needs_escape, string))
