@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2015 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -13,6 +12,8 @@ and under which environment.
 import os
 import sys
 import ctypes
+import configparser
+import fnmatch
 
 from gi.repository import GLib, Gio
 
@@ -32,6 +33,23 @@ def is_flatpak():
     return is_linux() and os.path.exists("/.flatpak-info")
 
 
+def matches_flatpak_runtime(pattern: str) -> bool:
+    """Pass a fnmatch pattern for matching the flatpak runtime ID"""
+
+    if not is_linux():
+        return False
+
+    config = configparser.ConfigParser()
+    try:
+        with open("/.flatpak-info", "r", encoding="utf-8") as f:
+            config.read_file(f)
+        runtime = config.get("Application", "runtime")
+    except (OSError, configparser.Error):
+        return False
+
+    return fnmatch.fnmatchcase(runtime, pattern)
+
+
 def is_plasma():
     """If we are running under plasma"""
 
