@@ -1,5 +1,6 @@
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #           2016-2020 Nick Boultbee
+#                2022 Jej@github
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +30,7 @@ from quodlibet.qltk.views import HintedTreeView
 from quodlibet.qltk.window import UniqueWindow, PersistentWindowMixin
 from quodlibet.qltk.x import Align, Paned, Button, ScrolledWindow
 from quodlibet.util import connect_obj
+from quodlibet.util.string.filter import remove_diacritics
 
 PLUGIN_CATEGORIES = {
     _("Songs"): SongsMenuPlugin,
@@ -76,7 +78,7 @@ class PluginErrorWindow(UniqueWindow):
         keys = failures.keys()
         show_expanded = len(keys) <= 3
         for key in sorted(keys):
-            expander = Gtk.Expander(label="<b>%s</b>" % util.escape(key))
+            expander = Gtk.Expander(label=util.bold(key))
             expander.set_use_markup(True)
             if show_expanded:
                 expander.set_expanded(True)
@@ -139,7 +141,7 @@ class PluginEnabledFilterCombo(Gtk.ComboBox):
         active = max(self.get_active(), 0)
         combo_store = self.get_model()
         combo_store.clear()
-        combo_store.append([_("All"), EnabledType.ALL])
+        combo_store.append([_("Any state"), EnabledType.ALL])
         combo_store.append(["", EnabledType.SEP])
         combo_store.append([_("Enabled"), EnabledType.EN])
         combo_store.append([_("Disabled"), EnabledType.DIS])
@@ -181,7 +183,7 @@ class PluginTypeFilterCombo(Gtk.ComboBox):
         active = max(self.get_active(), 0)
         combo_store = self.get_model()
         combo_store.clear()
-        combo_store.append([_("All"), object])
+        combo_store.append([_("Any category"), object])
         combo_store.append(["", None])
         for name, cls in PLUGIN_CATEGORIES.items():
             combo_store.append([name, cls])
@@ -306,6 +308,8 @@ class PluginPreferencesContainer(Gtk.VBox):
 
         self.desc = desc = Gtk.Label()
         desc.set_line_wrap(True)
+        # Ensure a reasonable minimum height request for long descriptions
+        desc.set_width_chars(30)
         desc.set_alignment(0, 0.5)
         desc.set_selectable(True)
         self.pack_start(desc, False, True, 0)
@@ -329,9 +333,9 @@ class PluginPreferencesContainer(Gtk.VBox):
             text = (f"<big><b>{name}</b> "
                     f"<span alpha='40%'> – {category}</span>"
                     f"</big>")
-            if plugin.description:
-                text += "<span font='4'>\n\n</span>"
-                text += util.escape(plugin.description)
+            markup = plugin.description_markup
+            if markup:
+                text += f"<span font='4'>\n\n</span>{markup}"
             label.set_markup(text)
             label.connect("activate-link", show_uri)
 
@@ -495,10 +499,12 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
                 return False
 
         def matches(text, filter_):
-            return all(p in text.lower() for p in filter_.lower().split())
+            return all(p in remove_diacritics(text.lower())
+                       for p in filter_.lower().split())
 
-        filter_ = entry.get_text()
+        filter_ = remove_diacritics(entry.get_text())
         return (matches(plugin.name, filter_) or
+                matches(plugin.id, filter_) or
                 matches((plugin.description or ""), filter_))
 
     def __destroy(self, *args):
