@@ -7,8 +7,11 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+from typing import Any, Iterable, List, Optional, Sequence, Type
+
 from gi.repository import Gtk
 
+from quodlibet.order import Order
 from quodlibet.qltk.playorder import OrderInOrder
 from quodlibet.qltk.models import ObjectStore
 from quodlibet.util import print_d
@@ -119,8 +122,9 @@ class PlaylistMux:
             assert source in (self.pl, self.q)
             if source is self.q:
                 main, other = other, main
-        other.go_to(None)
         res = main.go_to(song, explicit)
+        if res is not None or not explicit:
+            other.go_to(None)
         self._check_sourced()
         return res
 
@@ -150,10 +154,10 @@ class TrackCurrentModel(ObjectStore):
         super().__init__(*args, **kwargs)
         self.__iter = None
 
-    last_current = None
+    last_current: Optional[Any] = None
     """The last valid current song"""
 
-    def set(self, songs):
+    def set(self, songs: Sequence[Any]):
         """Clear the model and add the passed songs"""
 
         print_d("Filling view model with %d songs." % len(songs))
@@ -165,13 +169,13 @@ class TrackCurrentModel(ObjectStore):
             if song is oldsong:
                 self.__iter = iter_
 
-    def get(self):
+    def get(self) -> List[Any]:
         """A list of all contained songs"""
 
         return list(self.itervalues())
 
     @property
-    def current(self):
+    def current(self) -> Optional[Any]:
         """The current song or None"""
 
         return self.__iter and self.get_value(self.__iter, 0)
@@ -199,7 +203,7 @@ class TrackCurrentModel(ObjectStore):
         self.__iter = iter_
         self.last_current = self.current
 
-    def find(self, song):
+    def find(self, song: Any):
         """Returns the iter to the first occurrence of song in the model
         or None if it wasn't found.
         """
@@ -214,18 +218,13 @@ class TrackCurrentModel(ObjectStore):
                 return iter_
         return
 
-    def find_all(self, songs):
+    def find_all(self, songs: Iterable[Any]):
         """Returns a list of iters for all occurrences of all songs.
         (since a song can be in the model multiple times)
         """
 
         songs = set(songs)
-        found = []
-        append = found.append
-        for iter_, value in self.iterrows():
-            if value in songs:
-                append(iter_)
-        return found
+        return [iter_ for iter_, value in self.iterrows() if value in songs]
 
     def remove(self, iter_):
         if self.__iter and self[iter_].path == self[self.__iter].path:
@@ -243,23 +242,16 @@ class TrackCurrentModel(ObjectStore):
 class PlaylistModel(TrackCurrentModel):
     """A play list model for song lists"""
 
-    order = None
-    """The active `PlayOrder`"""
+    order: Order
+    """The active play order"""
 
     sourced = False
     """True in case this model is the source of the currently playing song"""
 
-    def __init__(self, order_cls=OrderInOrder):
+    def __init__(self, order_cls: Type[Order] = OrderInOrder):
         super().__init__(object)
         self.order = order_cls()
 
-        # The playorder plugins use paths atm to remember songs so
-        # we need to reset them if the paths change somehow.
-        self.__sigs = []
-        for sig in ['row-deleted', 'row-inserted', 'rows-reordered']:
-            s = self.connect(sig, lambda pl, *x: self.order.reset(pl))
-            self.__sigs.append(s)
-
     def next(self):
         """Switch to the next song"""
 
@@ -310,15 +302,11 @@ class PlaylistModel(TrackCurrentModel):
 
         return self.current_iter
 
-    def set(self, songs):
+    def set(self, songs: Sequence[Any]):
         """Clear the model and add the passed songs"""
 
         self.order.reset(self)
-        for signal_id in self.__sigs:
-            self.handler_block(signal_id)
         super().set(songs)
-        for signal_id in self.__sigs:
-            self.handler_unblock(signal_id)
 
     def reset(self):
         """Switch to the first song"""
