@@ -1,21 +1,24 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import re
 import os
 
 from gi.repository import Gtk
+from senf import fsn2text
 
 import quodlibet
 
+from quodlibet import _, ngettext
 from quodlibet import config
 from quodlibet import qltk
 from quodlibet import util
 
+from quodlibet.formats import AudioFileError
 from quodlibet.plugins import PluginManager
 from quodlibet.qltk._editutils import FilterPluginBox, FilterCheckButton
 from quodlibet.qltk._editutils import EditingPluginHandler, OverwriteWarning
@@ -25,10 +28,10 @@ from quodlibet.qltk.views import TreeViewColumn
 from quodlibet.qltk.cbes import ComboBoxEntrySave
 from quodlibet.qltk.models import ObjectStore
 from quodlibet.qltk import Icons
-from quodlibet.util.path import fsdecode
 from quodlibet.util.tagsfrompath import TagsFromPattern
 from quodlibet.util.string.splitters import split_value
-from quodlibet.util import connect_obj, gdecode
+from quodlibet.util import connect_obj
+from quodlibet.plugins.editing import TagsFromPathPlugin
 
 
 TBP = os.path.join(quodlibet.get_user_dir(), "lists", "tagpatterns")
@@ -67,17 +70,16 @@ class SplitTag(FilterCheckButton):
     _order = 1.2
 
     def filter(self, tag, value):
-        spls = config.get("editing", "split_on").decode('utf-8', 'replace')
+        spls = config.gettext("editing", "split_on")
         spls = spls.split()
         return "\n".join(split_value(value, spls))
 
 
 class TagsFromPathPluginHandler(EditingPluginHandler):
-    from quodlibet.plugins.editing import TagsFromPathPlugin
     Kind = TagsFromPathPlugin
 
 
-class ListEntry(object):
+class ListEntry:
 
     def __init__(self, song):
         self.song = song
@@ -91,7 +93,7 @@ class ListEntry(object):
 
     @property
     def name(self):
-        return fsdecode(self.song("~basename"))
+        return fsn2text(self.song("~basename"))
 
 
 class TagsFromPath(Gtk.VBox):
@@ -104,7 +106,7 @@ class TagsFromPath(Gtk.VBox):
         PluginManager.instance.register_handler(cls.handler)
 
     def __init__(self, parent, library):
-        super(TagsFromPath, self).__init__(spacing=6)
+        super().__init__(spacing=6)
 
         self.set_border_width(12)
         hbox = Gtk.HBox(spacing=6)
@@ -147,7 +149,7 @@ class TagsFromPath(Gtk.VBox):
         self.pack_start(filter_box, False, True, 0)
 
         # Save button
-        self.save = qltk.Button(_("Save"), Icons.DOCUMENT_SAVE)
+        self.save = qltk.Button(_("_Save"), Icons.DOCUMENT_SAVE)
         self.save.show()
         bbox = Gtk.HButtonBox()
         bbox.set_layout(Gtk.ButtonBoxStyle.END)
@@ -181,7 +183,7 @@ class TagsFromPath(Gtk.VBox):
             songs = [row[0].song for row in (self.view.get_model() or [])]
 
         if songs:
-            pattern_text = gdecode(self.combo.get_child().get_text())
+            pattern_text = self.combo.get_child().get_text()
         else:
             pattern_text = ""
         try:
@@ -189,10 +191,10 @@ class TagsFromPath(Gtk.VBox):
         except re.error:
             qltk.ErrorMessage(
                 self, _("Invalid pattern"),
-                _("The pattern\n\t<b>%s</b>\nis invalid. "
+                _("The pattern\n\t%s\nis invalid. "
                   "Possibly it contains the same tag twice or "
                   "it has unbalanced brackets (&lt; / &gt;).") % (
-                util.escape(pattern_text))).run()
+                util.bold(pattern_text)), escape_desc=False).run()
             return
         else:
             if pattern_text:
@@ -204,17 +206,15 @@ class TagsFromPath(Gtk.VBox):
         for header in pattern.headers:
             if not min([song.can_change(header) for song in songs]):
                 invalid.append(header)
-        if len(invalid) and songs:
-            if len(invalid) == 1:
-                title = _("Invalid tag")
-                msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
-                        " selected do not support editing this tag.")
-            else:
-                title = _("Invalid tags")
-                msg = _("Invalid tags <b>%s</b>\n\nThe files currently"
-                        " selected do not support editing these tags.")
-            qltk.ErrorMessage(
-                self, title, msg % ", ".join(invalid)).run()
+        total = len(invalid)
+        if total and songs:
+            title = ngettext("Invalid tag", "Invalid tags", total)
+            msg = ngettext("Invalid tag %s\n\nThe files currently "
+                           "selected do not support editing this tag.",
+                           "Invalid tags %s\n\nThe files currently "
+                           "selected do not support editing these tags.", total)
+            tags_str = util.bold(", ".join(invalid))
+            qltk.ErrorMessage(self, title, msg % tags_str, escape_desc=False).run()
             pattern = TagsFromPattern("")
 
         self.view.set_model(None)
@@ -268,7 +268,7 @@ class TagsFromPath(Gtk.VBox):
         self.save.set_sensitive(len(pattern.headers) > 0)
 
     def __save(self, addreplace, library):
-        pattern_text = gdecode(self.combo.get_child().get_text())
+        pattern_text = self.combo.get_child().get_text()
         pattern = TagsFromPattern(pattern_text)
         model = self.view.get_model()
         add = bool(addreplace.get_active())
@@ -278,7 +278,7 @@ class TagsFromPath(Gtk.VBox):
         was_changed = set()
 
         all_done = False
-        for entry in ((model and model.itervalues()) or []):
+        for entry in ((model and model.values()) or []):
             song = entry.song
             changed = False
             if not song.valid():
@@ -305,7 +305,7 @@ class TagsFromPath(Gtk.VBox):
             if changed:
                 try:
                     song.write()
-                except:
+                except AudioFileError:
                     util.print_exc()
                     WriteFailedError(self, song).run()
                     library.reload(song, changed=was_changed)
@@ -323,7 +323,6 @@ class TagsFromPath(Gtk.VBox):
 
     def __row_edited(self, renderer, path, new, model, header):
         entry = model[path][0]
-        new = gdecode(new)
         if entry.get_match(header) != new:
             entry.replace_match(header, new)
             self.preview.set_sensitive(True)
