@@ -10,7 +10,7 @@ import sys
 import gettext
 import locale
 
-from senf import environ, path2fsn, fsn2text, text2fsn
+from senf import path2fsn, fsn2text, text2fsn
 
 from quodlibet.util.path import unexpand, xdg_get_system_data_dirs
 from quodlibet.util.dprint import print_d
@@ -72,18 +72,18 @@ def set_i18n_envvars():
                                       [GetUserDefaultUILanguage(),
                                        GetSystemDefaultUILanguage()])))
         if langs:
-            environ.setdefault('LANG', langs[0])
-            environ.setdefault('LANGUAGE', ":".join(langs))
+            os.environ.setdefault('LANG', langs[0])
+            os.environ.setdefault('LANGUAGE', ":".join(langs))
     elif sys.platform == "darwin":
         from AppKit import NSLocale
         locale_id = NSLocale.currentLocale().localeIdentifier()
         lang = osx_locale_id_to_lang(locale_id)
-        environ.setdefault('LANG', lang)
+        os.environ.setdefault('LANG', lang)
 
         preferred_langs = NSLocale.preferredLanguages()
         if preferred_langs:
             languages = map(bcp47_to_language, preferred_langs)
-            environ.setdefault('LANGUAGE', ":".join(languages))
+            os.environ.setdefault('LANGUAGE', ":".join(languages))
     else:
         return
 
@@ -99,7 +99,7 @@ def fixup_i18n_envvars():
     """
 
     try:
-        langs = environ["LANGUAGE"].split(":")
+        langs = os.environ["LANGUAGE"].split(":")
     except KeyError:
         return
 
@@ -112,7 +112,7 @@ def fixup_i18n_envvars():
         if lang.startswith("en") and len(langs) > 1:
             sanitized.append("C")
 
-    environ["LANGUAGE"] = ":".join(sanitized)
+    os.environ["LANGUAGE"] = ":".join(sanitized)
 
 
 class GlibTranslations(gettext.GNUTranslations):
@@ -269,8 +269,8 @@ def init(language=None):
     set_i18n_envvars()
     fixup_i18n_envvars()
 
-    print_d("LANGUAGE: %r" % environ.get("LANGUAGE"))
-    print_d("LANG: %r" % environ.get("LANG"))
+    print_d("LANGUAGE: %r" % os.environ.get("LANGUAGE"))
+    print_d("LANG: %r" % os.environ.get("LANG"))
 
     try:
         locale.setlocale(locale.LC_ALL, '')
@@ -283,8 +283,8 @@ def init(language=None):
     assert "gi.repository.Gst" not in sys.modules
 
     if language is not None:
-        environ["LANGUAGE"] = text2fsn(language)
-        print_d("LANGUAGE: %r" % environ.get("LANGUAGE"))
+        os.environ["LANGUAGE"] = text2fsn(language)
+        print_d("LANGUAGE: %r" % os.environ.get("LANGUAGE"))
 
     _initialized = True
 
@@ -298,7 +298,7 @@ def get_available_languages(domain):
         Set[str]
     """
 
-    langs = set(["C"])
+    langs = {"C"}
 
     for locale_dir in iter_locale_dirs():
         try:
