@@ -74,7 +74,7 @@ def get_importables(folder):
                 dirs.remove(d)
         if not first and any((is_init(n) for n in names)):
             yield (basename(root), root,
-                   list(filter(is_ok, [join(root, name) for name in names])))
+                   [d for d in (join(root, name) for name in names) if is_ok(d)])
         else:
             for name in filter(is_ok, names):
                 yield (splitext(name)[0], join(root, name), [join(root, name)])
@@ -92,8 +92,8 @@ def load_module(name, package, path):
     except KeyError:
         pass
 
-    loader = importlib.find_loader(fullname, [path])
-    if loader is None:
+    spec = importlib.machinery.PathFinder.find_spec(fullname, [path])
+    if spec is None:
         return
 
     # modules need a parent package
@@ -101,7 +101,7 @@ def load_module(name, package, path):
         spec = importlib.machinery.ModuleSpec(package, None, is_package=True)
         sys.modules[package] = importlib.util.module_from_spec(spec)
 
-    mod = loader.load_module(fullname)
+    mod = spec.loader.load_module(fullname)
 
     # make it accessible from the parent, like __import__ does
     vars(sys.modules[package])[name] = mod
